@@ -21,6 +21,7 @@ package main
 
 import (
 	"bytes"
+	"context"
 	"crypto/subtle"
 	"encoding/json"
 	"errors"
@@ -108,8 +109,8 @@ var (
 
 	secbootMeasureSnapSystemEpochWhenPossible     func() error
 	secbootMeasureSnapModelWhenPossible           func(findModel func() (*asserts.Model, error)) error
-	secbootUnlockVolumeUsingSealedKeyIfEncrypted  func(disk disks.Disk, name string, encryptionKeyFile string, opts *secboot.UnlockVolumeUsingSealedKeyOptions) (secboot.UnlockResult, error)
-	secbootUnlockEncryptedVolumeUsingProtectorKey func(disk disks.Disk, name string, key []byte) (secboot.UnlockResult, error)
+	secbootUnlockVolumeUsingSealedKeyIfEncrypted  func(activation secboot.ActivateContext, disk disks.Disk, name string, sealedEncryptionKeyFiles []*secboot.LegacyKeyFile, opts *secboot.UnlockVolumeUsingSealedKeyOptions) (secboot.UnlockResult, error)
+	secbootUnlockEncryptedVolumeUsingProtectorKey func(activation secboot.ActivateContext, disk disks.Disk, name string, key []byte) (secboot.UnlockResult, error)
 
 	secbootLockSealedKeys func() error
 
@@ -126,6 +127,7 @@ var (
 	bootEnsureNextBootToRunMode      = boot.EnsureNextBootToRunMode
 	installBuildInstallObserver      = install.BuildInstallObserver
 	lookupDmVerityDataAndCrossCheck  = integrity.LookupDmVerityDataAndCrossCheck
+	secbootNewActivateContext        = secboot.NewActivateContext
 )
 
 func stampedAction(stamp string, action func() error) error {
@@ -170,9 +172,14 @@ func generateInitramfsMounts() (err error) {
 		return err
 	}
 
+	activationContext, err := secbootNewActivateContext(context.Background())
+	if err != nil {
+		return err
+	}
 	mst := &initramfsMountsState{
-		mode:           mode,
-		recoverySystem: recoverySystem,
+		mode:            mode,
+		recoverySystem:  recoverySystem,
+		activateContext: activationContext,
 	}
 	// generate mounts and set mst.validatedModel
 	switch mode {
@@ -874,6 +881,8 @@ type partitionState struct {
 }
 
 type diskUnlockState struct {
+	Activation secboot.ActivateContext
+
 	// UbuntuData is the state of the ubuntu-data (or ubuntu-data-enc)
 	// partition.
 	UbuntuData partitionState
@@ -891,6 +900,7 @@ func (r *diskUnlockState) serializeTo(name string) error {
 		UbuntuData: r.UbuntuData.PartitionState,
 		UbuntuBoot: r.UbuntuBoot.PartitionState,
 		UbuntuSave: r.UbuntuSave.PartitionState,
+		State:      r.Activation.State(),
 	}
 
 	return exportState.WriteTo(name)
@@ -945,6 +955,8 @@ type recoverModeStateMachine struct {
 	// state for tracking what happens as we progress through degraded mode of
 	// recovery
 	degradedState *diskUnlockState
+
+	activateContext secboot.ActivateContext
 }
 
 func (m *recoverModeStateMachine) whichModel() (*asserts.Model, error) {
@@ -1202,13 +1214,14 @@ func (m *recoverModeStateMachine) setUnlockStateWithFallbackKey(partName string,
 	return nil
 }
 
-func newRecoverModeStateMachine(model *asserts.Model, bootMode string, disk disks.Disk, allowFallback bool) *recoverModeStateMachine {
+func newRecoverModeStateMachine(activateContext secboot.ActivateContext, model *asserts.Model, bootMode string, disk disks.Disk, allowFallback bool) *recoverModeStateMachine {
 	m := &recoverModeStateMachine{
-		model:         model,
-		mode:          bootMode,
-		disk:          disk,
-		degradedState: &diskUnlockState{},
-		noFallback:    !allowFallback,
+		model:           model,
+		mode:            bootMode,
+		disk:            disk,
+		degradedState:   &diskUnlockState{Activation: activateContext},
+		noFallback:      !allowFallback,
+		activateContext: activateContext,
 	}
 	// first step is to mount ubuntu-boot to check for run mode keys to unlock
 	// ubuntu-data
@@ -1266,7 +1279,7 @@ func (m *recoverModeStateMachine) trustData() bool {
 
 // mountBoot is the first state to execute in the state machine, it can
 // transition to the following states:
-//   - if ubuntu-boot is mounted successfully, execute unlockDataRunKey
+//   - if ubuntu-boot is mounted successfully, execute unlockData
 //   - if ubuntu-boot can't be mounted, execute unlockDataFallbackKey
 //   - if we mounted the wrong ubuntu-boot (or otherwise can't verify which one we
 //     mounted), return fatal error
@@ -1280,10 +1293,7 @@ func (m *recoverModeStateMachine) mountBoot() (stateFunc, error) {
 		return nil, err
 	}
 	if part.findState != partitionFound {
-		// if we didn't find ubuntu-boot, we can't try to unlock data with the
-		// run key, and should instead just jump straight to attempting to
-		// unlock with the fallback key
-		return m.unlockDataFallbackKey, nil
+		return m.unlockData, nil
 	}
 
 	// should we fsck ubuntu-boot? probably yes because on some platforms
@@ -1298,14 +1308,9 @@ func (m *recoverModeStateMachine) mountBoot() (stateFunc, error) {
 	if err := m.setMountState("ubuntu-boot", boot.InitramfsUbuntuBootDir, mountErr); err != nil {
 		return nil, err
 	}
-	if part.MountState == boot.PartitionErrMounting {
-		// if we didn't mount data, then try to unlock data with the
-		// fallback key
-		return m.unlockDataFallbackKey, nil
-	}
 
 	// next step try to unlock data with run object
-	return m.unlockDataRunKey, nil
+	return m.unlockData, nil
 }
 
 // stateUnlockDataRunKey will try to unlock ubuntu-data with the normal run-mode
@@ -1313,67 +1318,48 @@ func (m *recoverModeStateMachine) mountBoot() (stateFunc, error) {
 // - failed to unlock data, but we know it's an encrypted device -> try to unlock with fallback key
 // - failed to find data at all -> try to unlock save
 // - unlocked data with run key -> mount data
-func (m *recoverModeStateMachine) unlockDataRunKey() (stateFunc, error) {
-	runModeKey := device.DataSealedKeyUnder(boot.InitramfsBootEncryptionKeyDir)
+func (m *recoverModeStateMachine) unlockData() (stateFunc, error) {
+	keys := []*secboot.LegacyKeyFile{
+		{
+			Name: "legacy",
+			Path: device.DataSealedKeyUnder(boot.InitramfsBootEncryptionKeyDir),
+		},
+	}
+	if !m.noFallback {
+		keys = append(keys, &secboot.LegacyKeyFile{
+			Name: "legacy-fallback",
+			Path: device.FallbackDataSealedKeyUnder(boot.InitramfsSeedEncryptionKeyDir),
+		})
+	}
+
 	unlockOpts := &secboot.UnlockVolumeUsingSealedKeyOptions{
-		// don't allow using the recovery key to unlock, we only try using the
-		// recovery key after we first try the fallback object
-		AllowRecoveryKey: false,
-		WhichModel:       m.whichModel,
-		BootMode:         m.mode,
-	}
-	unlockRes, unlockErr := secbootUnlockVolumeUsingSealedKeyIfEncrypted(m.disk, "ubuntu-data", runModeKey, unlockOpts)
-	m.setUnlockStateWithRunKey("ubuntu-data", unlockRes, unlockErr)
-	if unlockErr != nil {
-		// we couldn't unlock ubuntu-data with the primary key, or we didn't
-		// find it in the unencrypted case
-		if unlockRes.IsEncrypted {
-			// we know the device is encrypted, so the next state is to try
-			// unlocking with the fallback key
-			return m.unlockDataFallbackKey, nil
-		}
-
-		// if we didn't even find the device to the point where it would have
-		// been identified as decrypted or unencrypted device, we could have
-		// just entirely lost ubuntu-data-enc, and we could still have an
-		// encrypted device, so instead try to unlock ubuntu-save with the
-		// fallback key, the logic there can also handle an unencrypted ubuntu-save
-		return m.unlockMaybeEncryptedAloneSaveFallbackKey, nil
-	}
-
-	// otherwise successfully unlocked it (or just found it if it was unencrypted)
-	// so just mount it
-	return m.mountData, nil
-}
-
-func (m *recoverModeStateMachine) unlockDataFallbackKey() (stateFunc, error) {
-	if m.noFallback {
-		return nil, fmt.Errorf("cannot unlock ubuntu-data (fallback disabled)")
-	}
-
-	// try to unlock data with the fallback key on ubuntu-seed, which must have
-	// been mounted at this point
-	unlockOpts := &secboot.UnlockVolumeUsingSealedKeyOptions{
-		// we want to allow using the recovery key if the fallback key fails as
-		// using the fallback object is the last chance before we give up trying
-		// to unlock data
 		AllowRecoveryKey: true,
 		WhichModel:       m.whichModel,
 		BootMode:         m.mode,
 	}
-	// TODO: this prompts for a recovery key
-	// TODO: we should somehow customize the prompt to mention what key we need
-	// the user to enter, and what we are unlocking (as currently the prompt
-	// says "recovery key" and the partition UUID for what is being unlocked)
-	dataFallbackKey := device.FallbackDataSealedKeyUnder(boot.InitramfsSeedEncryptionKeyDir)
-	unlockRes, unlockErr := secbootUnlockVolumeUsingSealedKeyIfEncrypted(m.disk, "ubuntu-data", dataFallbackKey, unlockOpts)
-	if err := m.setUnlockStateWithFallbackKey("ubuntu-data", unlockRes, unlockErr); err != nil {
-		return nil, err
+	unlockRes, unlockErr := secbootUnlockVolumeUsingSealedKeyIfEncrypted(m.activateContext, m.disk, "ubuntu-data", keys, unlockOpts)
+	if unlockRes.Keyslot != "external:legacy-fallback" && unlockRes.Keyslot != "default-fallback" {
+		m.setUnlockStateWithRunKey("ubuntu-data", unlockRes, unlockErr)
+	} else {
+		if err := m.setUnlockStateWithFallbackKey("ubuntu-data", unlockRes, unlockErr); err != nil {
+			return nil, err
+		}
 	}
 	if unlockErr != nil {
-		// skip trying to mount data, since we did not unlock data we cannot
-		// open save with with the run key, so try the fallback one
-		return m.unlockEncryptedSaveFallbackKey, nil
+		if unlockRes.IsEncrypted {
+			// we know the device is encrypted, so the
+			// next state is to try unlocking encrypted
+			// save disk (and not look at unencrypte save
+			// disk).
+			// Future refactoring will merge
+			// unlockEncryptedSaveFallbackKey and
+			// unlockEncryptedSaveRunKey. But for
+			// now we can jump directly to the fallback
+			// since we cannot have a protector key.
+			return m.unlockEncryptedSaveFallbackKey, nil
+		} else {
+			return m.unlockMaybeEncryptedAloneSaveFallbackKey, nil
+		}
 	}
 
 	// unlocked it, now go mount it
@@ -1422,7 +1408,7 @@ func (m *recoverModeStateMachine) unlockEncryptedSaveRunKey() (stateFunc, error)
 		return m.unlockEncryptedSaveFallbackKey, nil
 	}
 
-	unlockRes, unlockErr := secbootUnlockEncryptedVolumeUsingProtectorKey(m.disk, "ubuntu-save", key)
+	unlockRes, unlockErr := secbootUnlockEncryptedVolumeUsingProtectorKey(m.activateContext, m.disk, "ubuntu-save", key)
 	m.setUnlockStateWithRunKey("ubuntu-save", unlockRes, unlockErr)
 	if unlockErr != nil {
 		// failed to unlock with run key, try fallback key
@@ -1494,13 +1480,20 @@ func (m *recoverModeStateMachine) unlockEncryptedSaveFallbackKey() (stateFunc, e
 		WhichModel:       m.whichModel,
 		BootMode:         m.mode,
 	}
-	saveFallbackKey := device.FallbackSaveSealedKeyUnder(boot.InitramfsSeedEncryptionKeyDir)
+
+	keys := []*secboot.LegacyKeyFile{
+		{
+			Name: "legacy-fallback",
+			Path: device.FallbackSaveSealedKeyUnder(boot.InitramfsSeedEncryptionKeyDir),
+		},
+	}
+
 	// TODO: this prompts again for a recover key, but really this is the
 	// reinstall key we will prompt for
 	// TODO: we should somehow customize the prompt to mention what key we need
 	// the user to enter, and what we are unlocking (as currently the prompt
 	// says "recovery key" and the partition UUID for what is being unlocked)
-	unlockRes, unlockErr := secbootUnlockVolumeUsingSealedKeyIfEncrypted(m.disk, "ubuntu-save", saveFallbackKey, unlockOpts)
+	unlockRes, unlockErr := secbootUnlockVolumeUsingSealedKeyIfEncrypted(m.activateContext, m.disk, "ubuntu-save", keys, unlockOpts)
 	if err := m.setUnlockStateWithFallbackKey("ubuntu-save", unlockRes, unlockErr); err != nil {
 		return nil, err
 	}
@@ -1594,7 +1587,7 @@ func generateMountsModeRecover(mst *initramfsMountsState) error {
 
 	machine, err := func() (machine *recoverModeStateMachine, err error) {
 		// first state to execute is to unlock ubuntu-data with the run key
-		machine = newRecoverModeStateMachine(model, "recover", disk, allowFallback)
+		machine = newRecoverModeStateMachine(mst.activateContext, model, "recover", disk, allowFallback)
 		for {
 			finished, err := machine.execute()
 			// TODO: consider whether certain errors are fatal or not
@@ -1735,7 +1728,7 @@ func generateMountsModeFactoryReset(mst *initramfsMountsState) error {
 	// invoked)
 	machine, err := func() (machine *recoverModeStateMachine, err error) {
 		allowFallback := true
-		machine = newRecoverModeStateMachine(model, "factory-reset", disk, allowFallback)
+		machine = newRecoverModeStateMachine(mst.activateContext, model, "factory-reset", disk, allowFallback)
 		// start from looking up encrypted ubuntu-save and unlocking with the fallback key
 		machine.current = machine.unlockMaybeEncryptedAloneSaveFallbackKey
 		for {
@@ -2180,7 +2173,7 @@ func generateMountsRecoverOrFactoryReset(mst *initramfsMountsState) (model *asse
 	return model, snaps, nil
 }
 
-func maybeMountSave(disk disks.Disk, rootdir string, encrypted bool, mountOpts *systemdMountOptions) (haveSave bool, unlockRes secboot.UnlockResult, err error) {
+func maybeMountSave(activateContext secboot.ActivateContext, disk disks.Disk, rootdir string, encrypted bool, mountOpts *systemdMountOptions) (haveSave bool, unlockRes secboot.UnlockResult, err error) {
 	var saveDevice string
 	if encrypted {
 		saveKey := device.SaveKeyUnder(dirs.SnapFDEDirUnder(rootdir))
@@ -2195,7 +2188,7 @@ func maybeMountSave(disk disks.Disk, rootdir string, encrypted bool, mountOpts *
 		if err != nil {
 			return true, unlockRes, err
 		}
-		unlockRes, err = secbootUnlockEncryptedVolumeUsingProtectorKey(disk, "ubuntu-save", key)
+		unlockRes, err = secbootUnlockEncryptedVolumeUsingProtectorKey(activateContext, disk, "ubuntu-save", key)
 		if err != nil {
 			return true, unlockRes, fmt.Errorf("cannot unlock ubuntu-save volume: %v", err)
 		}
@@ -2453,7 +2446,7 @@ func generateMountsModeRun(mst *initramfsMountsState) error {
 		return err
 	}
 
-	diskState := &diskUnlockState{}
+	diskState := &diskUnlockState{Activation: mst.activateContext}
 
 	// at this point on a system with TPM-based encryption
 	// data can be open only if the measured model matches the actual
@@ -2463,13 +2456,18 @@ func generateMountsModeRun(mst *initramfsMountsState) error {
 	// and we continue booting only for expected models
 
 	// 3.1. mount Data
-	runModeKey := device.DataSealedKeyUnder(boot.InitramfsBootEncryptionKeyDir)
+	keys := []*secboot.LegacyKeyFile{
+		{
+			Name: "legacy",
+			Path: device.DataSealedKeyUnder(boot.InitramfsBootEncryptionKeyDir),
+		},
+	}
 	opts := &secboot.UnlockVolumeUsingSealedKeyOptions{
 		AllowRecoveryKey: true,
 		WhichModel:       mst.UnverifiedBootModel,
 		BootMode:         mst.mode,
 	}
-	unlockRes, err := secbootUnlockVolumeUsingSealedKeyIfEncrypted(disk, "ubuntu-data", runModeKey, opts)
+	unlockRes, err := secbootUnlockVolumeUsingSealedKeyIfEncrypted(mst.activateContext, disk, "ubuntu-data", keys, opts)
 	if err != nil {
 		return err
 	}
@@ -2501,7 +2499,7 @@ func generateMountsModeRun(mst *initramfsMountsState) error {
 		NoSuid:    true,
 		NoExec:    true,
 	}
-	haveSave, saveUnlockRes, err := maybeMountSave(disk, rootfsDir, isEncryptedDev, saveMountOpts)
+	haveSave, saveUnlockRes, err := maybeMountSave(mst.activateContext, disk, rootfsDir, isEncryptedDev, saveMountOpts)
 	if err != nil {
 		return err
 	}
