@@ -280,7 +280,12 @@ func generateMountsModeRunCVM(mst *initramfsMountsState) error {
 		var unlockRes secboot.UnlockResult
 
 		if !pm.Opts.Tmpfs {
-			runModeCVMKey := filepath.Join(boot.InitramfsSeedEncryptionKeyDir, pm.GptLabel+".sealed-key")
+			keys := []*secboot.LegacyKeyFile{
+				{
+					Name: "legacy",
+					Path: filepath.Join(boot.InitramfsSeedEncryptionKeyDir, pm.GptLabel+".sealed-key"),
+				},
+			}
 			opts := &secboot.UnlockVolumeUsingSealedKeyOptions{
 				AllowRecoveryKey: true,
 			}
@@ -288,7 +293,7 @@ func generateMountsModeRunCVM(mst *initramfsMountsState) error {
 			// not the GPT label. Images that are created for CVM mode set both to the same label. The GPT label
 			// is used for partition discovery and the filesystem label for auto-discovery of a potentially encrypted
 			// partition.
-			unlockRes, err = secbootUnlockVolumeUsingSealedKeyIfEncrypted(disk, pm.GptLabel, runModeCVMKey, opts)
+			unlockRes, err = secbootUnlockVolumeUsingSealedKeyIfEncrypted(mst.activateContext, disk, pm.GptLabel, keys, opts)
 			if err != nil {
 				return err
 			}
