@@ -1685,6 +1685,20 @@ func (x *cmdRun) runSnapConfine(info *snap.Info, runner runnable, beforeExec fun
 		env["KRB5CCNAME"] = krb5ccnamePath
 	}
 
+	// Guarantee that XDG_RUNTIME_DIR does exist before launching the snap.
+	// XDG_RUNTIME_DIR has already been updated to point to a snap specific directory
+	// under the original $XDG_RUNTIME_DIR.
+	if xdg_runtime_dir, ok := env["XDG_RUNTIME_DIR"]; ok {
+		if err = os.Mkdir(xdg_runtime_dir, 0700); err != nil {
+			// The directory may already exist if snap was ran before, or one of the
+			// parent path components may not exist if /run/user/<uid>, which we
+			// expect as the path prefix, was not created during session setup.
+			if !errors.Is(err, os.ErrExist) && !errors.Is(err, os.ErrNotExist) {
+				return fmt.Errorf("cannot create XDG_RUNTIME_DIR folder %q: %w", xdg_runtime_dir, err)
+			}
+		}
+	}
+
 	// on each run variant path this will be used once to get
 	// the environment plus additions in the right form
 	envForExec := func(extra map[string]string) []string {
