@@ -37,12 +37,14 @@ import (
 	"github.com/snapcore/snapd/overlord/auth"
 	"github.com/snapcore/snapd/overlord/configstate/config"
 	"github.com/snapcore/snapd/overlord/devicestate"
+	"github.com/snapcore/snapd/overlord/fdestate"
 	"github.com/snapcore/snapd/overlord/ifacestate"
 	"github.com/snapcore/snapd/overlord/snapstate"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/release"
 	"github.com/snapcore/snapd/sandbox"
 	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/snapdtool"
 )
 
 var (
@@ -62,6 +64,16 @@ var (
 		WriteAccess: openAccess{},
 	}
 
+	sysInfoStorageEncCmd = &Command{
+		Path: "/v2/system-info/storage-encrypted",
+		GET:  sysInfoStorageEnc,
+		// The "status" field is publicly accessible. If additional sensitive fields
+		// are added in the future, they must not be included in the response unless
+		// the caller satisfies the required access constraints. Any such filtering
+		// is performed at a lower level.
+		ReadAccess: openAccess{},
+	}
+
 	stateChangeCmd = &Command{
 		Path:        "/v2/changes/{id}",
 		GET:         getChange,
@@ -88,8 +100,10 @@ var (
 )
 
 var (
-	buildID     = "unknown"
-	systemdVirt = ""
+	buildID             = "unknown"
+	systemdVirt         = ""
+	snapdtoolIsReexecd  = snapdtool.IsReexecd
+	fdestateSystemState = fdestate.SystemState
 )
 
 func init() {
@@ -127,6 +141,16 @@ func sysInfo(c *Command, r *http.Request, user *auth.UserState) Response {
 		return InternalError("cannot get user auth data: %s", err)
 	}
 
+	reexecd, err := snapdtoolIsReexecd()
+	if err != nil {
+		return InternalError("cannot obtain snapd reexec status: %s", err)
+	}
+
+	snapdFrom := "snap"
+	if !reexecd {
+		snapdFrom = "native-package"
+	}
+
 	refreshInfo := client.RefreshInfo{
 		Last: formatRefreshTime(lastRefresh),
 		Hold: formatRefreshTime(refreshHold),
@@ -150,10 +174,11 @@ func sysInfo(c *Command, r *http.Request, user *auth.UserState) Response {
 			"snap-mount-dir": dirs.SnapMountDir,
 			"snap-bin-dir":   dirs.SnapBinariesDir,
 		},
-		"refresh":      refreshInfo,
-		"architecture": arch.DpkgArchitecture(),
-		"system-mode":  deviceMgr.SystemMode(devicestate.SysAny),
-		"features":     features.All(tr),
+		"refresh":        refreshInfo,
+		"architecture":   arch.DpkgArchitecture(),
+		"system-mode":    deviceMgr.SystemMode(devicestate.SysAny),
+		"features":       features.All(tr),
+		"snapd-bin-from": snapdFrom,
 	}
 	if systemdVirt != "" {
 		m["virtualization"] = systemdVirt
@@ -541,3 +566,16 @@ func ackWarnings(c *Command, r *http.Request, _ *auth.UserState) Response {
 
 	return SyncResponse(n)
 }
+
+func sysInfoStorageEnc(c *Command, r *http.Request, user *auth.UserState) Response {
+	st := c.d.overlord.State()
+	st.Lock()
+	defer st.Unlock()
+
+	state, err := fdestateSystemState(st)
+	if err != nil {
+		return InternalError("cannot determine system encrypted state: %s", err)
+	}
+
+	return SyncResponse(state)
+}
