@@ -22,6 +22,7 @@ package daemon_test
 import (
 	"bytes"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"io"
 	"net/http"
@@ -41,6 +42,7 @@ import (
 	"github.com/snapcore/snapd/interfaces/ifacetest"
 	"github.com/snapcore/snapd/overlord/auth"
 	"github.com/snapcore/snapd/overlord/configstate/config"
+	"github.com/snapcore/snapd/overlord/fdestate"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/release"
 	"github.com/snapcore/snapd/sandbox"
@@ -65,6 +67,14 @@ func (s *generalSuite) expectChangesReadAccess() {
 	s.expectReadAccess(daemon.InterfaceOpenAccess{Interfaces: []string{"snap-refresh-observe"}})
 }
 
+func (s *generalSuite) expectChangeReadAccess() {
+	s.expectReadAccess(daemon.InterfaceOpenAccess{Interfaces: []string{"snap-refresh-observe", "ros-snapd-support"}})
+}
+
+func (s *generalSuite) expectSystemInfoStorageEncReadAccess() {
+	s.expectReadAccess(daemon.OpenAccess{})
+}
+
 func (s *generalSuite) TestRoot(c *check.C) {
 	s.daemon(c)
 
@@ -159,6 +169,7 @@ func (s *generalSuite) TestSysInfo(c *check.C) {
 		"architecture":     arch.DpkgArchitecture(),
 		"virtualization":   "magic",
 		"system-mode":      "run",
+		"snapd-bin-from":   "native-package",
 	}
 	var rsp daemon.RespJSON
 	c.Assert(json.Unmarshal(rec.Body.Bytes(), &rsp), check.IsNil)
@@ -289,6 +300,7 @@ func (s *generalSuite) TestSysInfoLegacyRefresh(c *check.C) {
 		"architecture":   arch.DpkgArchitecture(),
 		"virtualization": "kvm",
 		"system-mode":    "run",
+		"snapd-bin-from": "native-package",
 	}
 	var rsp daemon.RespJSON
 	c.Assert(json.Unmarshal(rec.Body.Bytes(), &rsp), check.IsNil)
@@ -301,6 +313,84 @@ func (s *generalSuite) TestSysInfoLegacyRefresh(c *check.C) {
 	c.Check(rsp.Result, check.DeepEquals, expected)
 }
 
+func (s *generalSuite) testSysInfoBinOrigin(c *check.C, exp string, expErr string) {
+	s.expectSystemInfoReadAccess()
+	req, err := http.NewRequest("GET", "/v2/system-info", nil)
+	c.Assert(err, check.IsNil)
+
+	restore := release.MockReleaseInfo(&release.OS{ID: "distro-id", VersionID: "1.2"})
+	defer restore()
+	restore = release.MockOnClassic(true)
+	defer restore()
+	restore = sandbox.MockForceDevMode(true)
+	defer restore()
+
+	r := c.MkDir()
+	// using unknown distro, set up
+	dirstest.MustMockAltSnapMountDir(r)
+	dirstest.MustMockClassicConfinementAltDirSupport(r)
+	// reload dirs for release info to have effect
+	dirs.SetRootDir(r)
+
+	restore = daemon.MockSystemdVirt("magic")
+	defer restore()
+	// Set systemd version <230 so QuotaGroups feature unsupported
+	restore = systemd.MockSystemdVersion(229, nil)
+	defer restore()
+
+	buildID := "this-is-my-build-id"
+	restore = daemon.MockBuildID(buildID)
+	defer restore()
+
+	d := s.daemon(c)
+	d.Version = "42b1"
+
+	s.expectSystemInfoReadAccess()
+
+	rec := httptest.NewRecorder()
+	s.req(c, req, nil, actionIsExpected).ServeHTTP(rec, nil)
+	if expErr == "" {
+		c.Check(rec.Code, check.Equals, 200)
+		c.Check(rec.Header().Get("Content-Type"), check.Equals, "application/json")
+
+		var rsp daemon.RespJSON
+		c.Assert(json.Unmarshal(rec.Body.Bytes(), &rsp), check.IsNil)
+		c.Check(rsp.Status, check.Equals, 200)
+		c.Check(rsp.Type, check.Equals, daemon.ResponseTypeSync)
+
+		m, _ := rsp.Result.(map[string]any)
+		c.Assert(m, check.NotNil)
+		c.Check(m["snapd-bin-from"], check.Equals, exp)
+	} else {
+		c.Check(rec.Code, check.Equals, 500)
+		c.Check(rec.Header().Get("Content-Type"), check.Equals, "application/json")
+		assertResponseBody(c, rec.Body, map[string]any{
+			"type":        "error",
+			"status-code": float64(500),
+			"status":      "Internal Server Error",
+			"result": map[string]any{
+				"message": `cannot obtain snapd reexec status: mock error`,
+			},
+		})
+	}
+}
+
+func (s *generalSuite) TestSysInfoBinOriginSnapd(c *check.C) {
+	defer daemon.MockSnapdtoolsIsReexecd(func() (bool, error) {
+		return true, nil
+	})()
+
+	s.testSysInfoBinOrigin(c, "snap", "")
+}
+
+func (s *generalSuite) TestSysInfoBinOriginError(c *check.C) {
+	defer daemon.MockSnapdtoolsIsReexecd(func() (bool, error) {
+		return false, errors.New("mock error")
+	})()
+
+	s.testSysInfoBinOrigin(c, "", "foo bar")
+}
+
 func (s *generalSuite) testSysInfoSystemMode(c *check.C, mode string) {
 	s.expectSystemInfoReadAccess()
 	req, err := http.NewRequest("GET", "/v2/system-info", nil)
@@ -368,8 +458,9 @@ func (s *generalSuite) testSysInfoSystemMode(c *check.C, mode string) {
 			"apparmor":            []any{"feature-1", "feature-2"},
 			"confinement-options": []any{"devmode", "strict"}, // we know it's this because of the release.Mock... calls above
 		},
-		"architecture": arch.DpkgArchitecture(),
-		"system-mode":  mode,
+		"architecture":   arch.DpkgArchitecture(),
+		"system-mode":    mode,
+		"snapd-bin-from": "native-package",
 	}
 	var rsp daemon.RespJSON
 	c.Assert(json.Unmarshal(rec.Body.Bytes(), &rsp), check.IsNil)
@@ -911,7 +1002,7 @@ func (s *generalSuite) TestStateChange(c *check.C) {
 	defer restore()
 
 	// Setup
-	s.expectChangesReadAccess()
+	s.expectChangeReadAccess()
 	d := s.daemon(c)
 	st := d.Overlord().State()
 	st.Lock()
@@ -997,7 +1088,7 @@ func (s *generalSuite) TestStateChangeAbort(c *check.C) {
 	defer restore()
 
 	// Setup
-	s.expectChangesReadAccess()
+	s.expectChangeReadAccess()
 	d := s.daemon(c)
 	st := d.Overlord().State()
 	st.Lock()
@@ -1063,7 +1154,7 @@ func (s *generalSuite) TestStateChangeAbortIsReady(c *check.C) {
 	defer restore()
 
 	// Setup
-	s.expectChangesReadAccess()
+	s.expectChangeReadAccess()
 	d := s.daemon(c)
 	st := d.Overlord().State()
 	st.Lock()
@@ -1140,3 +1231,66 @@ func (s *generalSuite) TestAckWarnings(c *check.C) {
 	c.Check(calls, check.Equals, "ok")
 	c.Check(result, check.DeepEquals, 0)
 }
+
+func (s *generalSuite) TestSysInfoStorageEncHappy(c *check.C) {
+	s.daemon(c)
+	s.expectSystemInfoStorageEncReadAccess()
+
+	expectedStatus := ""
+	expectedResponse := map[string]any{}
+
+	setExpectedStatus := func(status string) {
+		expectedStatus = status
+		expectedResponse["status"] = status
+	}
+
+	defer daemon.MockFdestateSystemState(func(*state.State) (*fdestate.FDESystemState, error) {
+		switch expectedStatus {
+		case "active":
+			return &fdestate.FDESystemState{Status: fdestate.FDEStatusActive}, nil
+
+		case "inactive":
+			return &fdestate.FDESystemState{Status: fdestate.FDEStatusInactive}, nil
+		}
+
+		return nil, errors.New("cannot set unsupported expected status")
+	})()
+
+	req, err := http.NewRequest("GET", "/v2/system-info/storage-encrypted", nil)
+	c.Assert(err, check.IsNil)
+
+	setExpectedStatus("active")
+	rsp := s.syncReq(c, req, nil, actionIsExpected)
+	c.Check(rsp.Status, check.Equals, 200)
+	c.Check(rsp.Type, check.Equals, daemon.ResponseTypeSync)
+	resultBytes, err := json.Marshal(rsp.Result)
+	c.Assert(err, check.IsNil)
+	var resultAbstract any
+	err = json.Unmarshal(resultBytes, &resultAbstract)
+	c.Assert(err, check.IsNil)
+	c.Check(resultAbstract, check.DeepEquals, expectedResponse)
+
+	setExpectedStatus("inactive")
+	rsp = s.syncReq(c, req, nil, actionIsExpected)
+	c.Assert(err, check.IsNil)
+	resultBytes, err = json.Marshal(rsp.Result)
+	c.Assert(err, check.IsNil)
+	err = json.Unmarshal(resultBytes, &resultAbstract)
+	c.Assert(err, check.IsNil)
+	c.Check(resultAbstract, check.DeepEquals, expectedResponse)
+}
+
+func (s *generalSuite) TestSysInfoStorageEncErrorImpl(c *check.C) {
+	s.daemon(c)
+
+	defer daemon.MockFdestateSystemState(func(*state.State) (*fdestate.FDESystemState, error) {
+		return nil, errors.New("cannot calculate status")
+	})()
+
+	req, err := http.NewRequest("GET", "/v2/system-info/storage-encrypted", nil)
+	c.Assert(err, check.IsNil)
+
+	rsp := s.errorReq(c, req, nil, actionIsExpected)
+	c.Check(rsp.Status, check.Equals, 500)
+	c.Check(rsp.Message, check.Equals, "cannot determine system encrypted state: cannot calculate status")
+}
