@@ -39,6 +39,7 @@ import (
 	"github.com/snapcore/snapd/overlord/snapstate/snapstatetest"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/release"
+	"github.com/snapcore/snapd/secboot"
 	"github.com/snapcore/snapd/secboot/keys"
 	"github.com/snapcore/snapd/snap/snaptest"
 )
@@ -58,6 +59,7 @@ func (s *systemVolumesSuite) SetUpTest(c *C) {
 			"check-passphrase":  daemon.InterfaceOpenAccess{Interfaces: []string{"snap-fde-control"}},
 			"check-pin":         daemon.InterfaceOpenAccess{Interfaces: []string{"snap-fde-control"}},
 			"change-passphrase": daemon.InterfaceOpenAccess{Interfaces: []string{"snap-fde-control"}},
+			"change-pin":        daemon.InterfaceOpenAccess{Interfaces: []string{"snap-fde-control"}},
 			"check-recovery-key": daemon.InterfaceRootAccess{
 				Interfaces: []string{"snap-fde-control", "firmware-updater-support"},
 				Polkit:     "io.snapcraft.snapd.manage-fde",
@@ -66,6 +68,10 @@ func (s *systemVolumesSuite) SetUpTest(c *C) {
 				Interfaces: []string{"snap-fde-control"},
 				Polkit:     "io.snapcraft.snapd.manage-fde",
 			},
+			"add-recovery-key": daemon.InterfaceRootAccess{
+				Interfaces: []string{"snap-fde-control"},
+				Polkit:     "io.snapcraft.snapd.manage-fde",
+			},
 			"replace-recovery-key": daemon.InterfaceRootAccess{
 				Interfaces: []string{"snap-fde-control"},
 				Polkit:     "io.snapcraft.snapd.manage-fde",
@@ -289,6 +295,164 @@ func (s *systemVolumesSuite) TestSystemVolumesActionCheckRecoveryKeyError(c *C)
 	c.Assert(rsp.Value, DeepEquals, map[string]any{"reason": fdestate.InvalidRecoveryKeyReasonInvalidValue})
 }
 
+func (s *systemVolumesSuite) TestSystemVolumesActionAddRecoveryKey(c *C) {
+	d := s.daemon(c)
+	s.mockHybridSystem()
+	st := d.Overlord().State()
+
+	d.Overlord().Loop()
+	defer d.Overlord().Stop()
+
+	called := 0
+	s.AddCleanup(daemon.MockFdestateAddRecoveryKey(func(st *state.State, recoveryKeyID string, keyslots []fdestate.KeyslotRef) (*state.TaskSet, error) {
+		called++
+		c.Check(recoveryKeyID, Equals, "some-key-id")
+		c.Check(keyslots, DeepEquals, []fdestate.KeyslotRef{
+			{ContainerRole: "some-container-role", Name: "some-name"},
+			// keyslot expanded
+			{ContainerRole: "system-data", Name: "some-other-name"},
+			{ContainerRole: "system-save", Name: "some-other-name"},
+		})
+
+		return state.NewTaskSet(st.NewTask("some-task", "")), nil
+	}))
+
+	body := strings.NewReader(`
+{
+	"action": "add-recovery-key",
+	"key-id": "some-key-id",
+	"keyslots": [
+		{"container-role": "some-container-role", "name": "some-name"},
+		{"name": "some-other-name"}
+	]
+}`)
+	req, err := http.NewRequest("POST", "/v2/system-volumes", body)
+	c.Assert(err, IsNil)
+	req.Header.Add("Content-Type", "application/json")
+
+	rsp := s.asyncReq(c, req, nil, actionIsExpected)
+	c.Assert(rsp.Status, Equals, 202)
+
+	st.Lock()
+	chg := st.Change(rsp.Change)
+	tsks := chg.Tasks()
+	st.Unlock()
+	c.Check(chg, NotNil)
+	c.Check(chg.ID(), Equals, "1")
+	c.Check(chg.Kind(), Equals, "fde-add-recovery-key")
+	c.Assert(tsks, HasLen, 1)
+	c.Check(tsks[0].Kind(), Equals, "some-task")
+	c.Check(called, Equals, 1)
+}
+
+func (s *systemVolumesSuite) TestSystemVolumesActionAddRecoveryKeyError(c *C) {
+	s.daemon(c)
+	s.mockHybridSystem()
+
+	var mockErr error
+	s.AddCleanup(daemon.MockFdestateAddRecoveryKey(func(st *state.State, recoveryKeyID string, keyslots []fdestate.KeyslotRef) (*state.TaskSet, error) {
+		return nil, mockErr
+	}))
+
+	// bad request error
+	body := strings.NewReader(`{"action": "add-recovery-key", "key-id": "some-key-id", "keyslots": [{"name": "some-keyslot"}]}`)
+	req, err := http.NewRequest("POST", "/v2/system-volumes", body)
+	c.Assert(err, IsNil)
+	req.Header.Add("Content-Type", "application/json")
+
+	mockErr = errors.New("boom!")
+	rsp := s.errorReq(c, req, nil, actionIsExpected)
+	c.Assert(rsp.Status, Equals, 400)
+	c.Check(rsp.Message, Equals, "cannot add recovery key: boom!")
+
+	// typed conflict detection error kind
+	mockErr = &snapstate.ChangeConflictError{
+		Message: fmt.Sprintf("conflict error: boom!"),
+	}
+	body = strings.NewReader(`{"action": "add-recovery-key", "key-id": "some-key-id", "keyslots": [{"name": "some-keyslot"}]}`)
+	req, err = http.NewRequest("POST", "/v2/system-volumes", body)
+	req.Header.Add("Content-Type", "application/json")
+
+	c.Assert(err, IsNil)
+	rsp = s.errorReq(c, req, nil, actionIsExpected)
+	c.Assert(rsp.Status, Equals, 409)
+	c.Check(rsp.Kind, Equals, client.ErrorKindSnapChangeConflict)
+	c.Check(rsp.Message, Equals, "conflict error: boom!")
+
+	// typed keyslots already exists error kind
+	mockErr = &fdestate.KeyslotsAlreadyExistsError{
+		Keyslots: []fdestate.Keyslot{{ContainerRole: "some-container-role", Name: "some-name"}},
+	}
+	body = strings.NewReader(`{"action": "add-recovery-key", "key-id": "some-key-id", "keyslots": [{"name": "some-keyslot"}]}`)
+	req, err = http.NewRequest("POST", "/v2/system-volumes", body)
+	req.Header.Add("Content-Type", "application/json")
+
+	c.Assert(err, IsNil)
+	rsp = s.errorReq(c, req, nil, actionIsExpected)
+	c.Assert(rsp.Status, Equals, 400)
+	c.Check(rsp.Kind, Equals, client.ErrorKindKeyslotsAlreadyExists)
+	c.Check(rsp.Message, Equals, `key slot (container-role: "some-container-role", name: "some-name") already exists`)
+	c.Check(rsp.Value, DeepEquals, []fdestate.KeyslotRef{{ContainerRole: "some-container-role", Name: "some-name"}})
+
+	// typed invalid recovery key error kind
+	mockErr = &fdestate.InvalidRecoveryKeyError{
+		Reason: fdestate.InvalidRecoveryKeyReasonNotFound,
+	}
+	body = strings.NewReader(`{"action": "add-recovery-key", "key-id": "some-key-id", "keyslots": [{"name": "some-keyslot"}]}`)
+	req, err = http.NewRequest("POST", "/v2/system-volumes", body)
+	req.Header.Add("Content-Type", "application/json")
+
+	c.Assert(err, IsNil)
+	rsp = s.errorReq(c, req, nil, actionIsExpected)
+	c.Assert(rsp.Status, Equals, 400)
+	c.Check(rsp.Kind, Equals, client.ErrorKindInvalidRecoveryKey)
+	c.Check(rsp.Message, Equals, `invalid recovery key: not found`)
+	c.Check(rsp.Value, DeepEquals, map[string]any{"reason": fdestate.InvalidRecoveryKeyReasonNotFound})
+
+	// typed insufficient container capacity error kind
+	mockErr = &fdestate.InsufficientContainerCapacityError{
+		ContainerRoles: []string{"system-data", "system-save"},
+	}
+	body = strings.NewReader(`{"action": "add-recovery-key", "key-id": "some-key-id", "keyslots": [{"name": "some-keyslot"}]}`)
+	req, err = http.NewRequest("POST", "/v2/system-volumes", body)
+	req.Header.Add("Content-Type", "application/json")
+
+	c.Assert(err, IsNil)
+	rsp = s.errorReq(c, req, nil, actionIsExpected)
+	c.Assert(rsp.Status, Equals, 400)
+	c.Check(rsp.Kind, Equals, client.ErrorKindInsufficientContainerCapacity)
+	c.Check(rsp.Message, Equals, `insufficient capacity on containers [system-data, system-save]`)
+	c.Check(rsp.Value, DeepEquals, []string{"system-data", "system-save"})
+}
+
+func (s *systemVolumesSuite) TestSystemVolumesActionAddRecoveryKeyMissingKeyID(c *C) {
+	s.daemon(c)
+	s.mockHybridSystem()
+
+	body := strings.NewReader(`{"action": "add-recovery-key"}`)
+	req, err := http.NewRequest("POST", "/v2/system-volumes", body)
+	c.Assert(err, IsNil)
+	req.Header.Add("Content-Type", "application/json")
+
+	rsp := s.errorReq(c, req, nil, actionIsExpected)
+	c.Assert(rsp.Status, Equals, 400)
+	c.Assert(rsp.Message, Equals, "system volume action requires key-id to be provided")
+}
+
+func (s *systemVolumesSuite) TestSystemVolumesActionAddRecoveryKeyMissingKeyslots(c *C) {
+	s.daemon(c)
+	s.mockHybridSystem()
+
+	body := strings.NewReader(`{"action": "add-recovery-key", "key-id": "some-id"}`)
+	req, err := http.NewRequest("POST", "/v2/system-volumes", body)
+	c.Assert(err, IsNil)
+	req.Header.Add("Content-Type", "application/json")
+
+	rsp := s.errorReq(c, req, nil, actionIsExpected)
+	c.Assert(rsp.Status, Equals, 400)
+	c.Assert(rsp.Message, Equals, "system volume action requires keyslots to be provided")
+}
+
 func (s *systemVolumesSuite) TestSystemVolumesActionReplaceRecoveryKey(c *C) {
 	d := s.daemon(c)
 	s.mockHybridSystem()
@@ -303,6 +467,9 @@ func (s *systemVolumesSuite) TestSystemVolumesActionReplaceRecoveryKey(c *C) {
 		c.Check(recoveryKeyID, Equals, "some-key-id")
 		c.Check(keyslots, DeepEquals, []fdestate.KeyslotRef{
 			{ContainerRole: "some-container-role", Name: "some-name"},
+			// keyslot expanded
+			{ContainerRole: "system-data", Name: "some-other-name"},
+			{ContainerRole: "system-save", Name: "some-other-name"},
 		})
 
 		return state.NewTaskSet(st.NewTask("some-task", "")), nil
@@ -313,7 +480,8 @@ func (s *systemVolumesSuite) TestSystemVolumesActionReplaceRecoveryKey(c *C) {
 	"action": "replace-recovery-key",
 	"key-id": "some-key-id",
 	"keyslots": [
-		{"container-role": "some-container-role", "name": "some-name"}
+		{"container-role": "some-container-role", "name": "some-name"},
+		{"name": "some-other-name"}
 	]
 }`)
 	req, err := http.NewRequest("POST", "/v2/system-volumes", body)
@@ -333,7 +501,6 @@ func (s *systemVolumesSuite) TestSystemVolumesActionReplaceRecoveryKey(c *C) {
 	c.Assert(tsks, HasLen, 1)
 	c.Check(tsks[0].Kind(), Equals, "some-task")
 	c.Check(called, Equals, 1)
-
 }
 
 func (s *systemVolumesSuite) TestSystemVolumesActionReplaceRecoveryKeyError(c *C) {
@@ -399,6 +566,38 @@ func (s *systemVolumesSuite) TestSystemVolumesActionReplaceRecoveryKeyError(c *C
 	c.Check(rsp.Kind, Equals, client.ErrorKindInvalidRecoveryKey)
 	c.Check(rsp.Message, Equals, `invalid recovery key: expired`)
 	c.Check(rsp.Value, DeepEquals, map[string]any{"reason": fdestate.InvalidRecoveryKeyReasonExpired})
+
+	for _, conflictingKeyslot := range []fdestate.KeyslotRef{
+		{ContainerRole: "some-container-role", Name: "explicit-duplicate"},
+		{ContainerRole: "system-data", Name: "implicit-duplicate"},
+		{ContainerRole: "system-save", Name: "implicit-duplicate"},
+	} {
+		bodyRaw := fmt.Sprintf(`{
+	"action": "replace-recovery-key",
+	"key-id": "some-key-id",
+	"keyslots": [
+		{"container-role": %q, "name": %q},
+		{"container-role": "some-container-role", "name": "explicit-duplicate"},
+		{"name": "implicit-duplicate"}
+	]
+}`, conflictingKeyslot.ContainerRole, conflictingKeyslot.Name)
+
+		body = strings.NewReader(bodyRaw)
+		req, err = http.NewRequest("POST", "/v2/system-volumes", body)
+		req.Header.Add("Content-Type", "application/json")
+
+		c.Assert(err, IsNil)
+		rsp = s.errorReq(c, req, nil, actionIsExpected)
+		c.Assert(rsp.Status, Equals, 400)
+		switch conflictingKeyslot.Name {
+		case "explicit-duplicate":
+			c.Check(rsp.Message, Equals, fmt.Sprintf("invalid keyslots: duplicate keyslot found %s", conflictingKeyslot.String()))
+		case "implicit-duplicate":
+			c.Check(rsp.Message, Equals, fmt.Sprintf("invalid keyslots: duplicate implicit keyslot found %s", conflictingKeyslot.String()))
+		default:
+			c.Errorf("unexpected keyslot name %q", conflictingKeyslot.Name)
+		}
+	}
 }
 
 func (s *systemVolumesSuite) TestSystemVolumesActionReplaceRecoveryKeyMissingKeyID(c *C) {
@@ -437,14 +636,17 @@ func (s *systemVolumesSuite) testSystemVolumesActionReplacePlatformKey(c *C, aut
 			})
 		case device.AuthModePIN:
 			c.Check(*volumesAuth, DeepEquals, device.VolumesAuthOptions{
-				// TODO:FDEM: check PIN is passed
 				Mode: device.AuthModePIN,
+				PIN:  "1234",
 			})
 		default:
 			c.Errorf("unexpected auth-mode %q", authMode)
 		}
 		c.Check(keyslotRefs, DeepEquals, []fdestate.KeyslotRef{
 			{ContainerRole: "some-container-role", Name: "some-name"},
+			// keyslot expanded
+			{ContainerRole: "system-data", Name: "some-other-name"},
+			{ContainerRole: "system-save", Name: "some-other-name"},
 		})
 
 		return state.NewTaskSet(st.NewTask("some-task", "")), nil
@@ -454,7 +656,8 @@ func (s *systemVolumesSuite) testSystemVolumesActionReplacePlatformKey(c *C, aut
 {
 	"action": "replace-platform-key",
 	"keyslots": [
-		{"container-role": "some-container-role", "name": "some-name"}
+		{"container-role": "some-container-role", "name": "some-name"},
+		{"name": "some-other-name"}
 	],
 %s
 }`
@@ -474,7 +677,7 @@ func (s *systemVolumesSuite) testSystemVolumesActionReplacePlatformKey(c *C, aut
 	case device.AuthModePIN:
 		bodyRaw = fmt.Sprintf(bodyTemplate, `
 	"auth-mode": "pin",
-	"pin": "p1n"
+	"pin": "1234"
 `)
 	}
 
@@ -483,14 +686,6 @@ func (s *systemVolumesSuite) testSystemVolumesActionReplacePlatformKey(c *C, aut
 	c.Assert(err, IsNil)
 	req.Header.Add("Content-Type", "application/json")
 
-	if authMode == device.AuthModePIN {
-		// PIN support is not implemented
-		rsp := s.errorReq(c, req, nil, actionIsExpected)
-		c.Assert(rsp.Status, Equals, 400)
-		c.Check(rsp.Message, Equals, `invalid platform key options: "pin" authentication mode is not implemented`)
-		return
-	}
-
 	rsp := s.asyncReq(c, req, nil, actionIsExpected)
 	c.Assert(rsp.Status, Equals, 202)
 
@@ -517,6 +712,10 @@ func (s *systemVolumesSuite) TestSystemVolumesActionReplacePlatformKeyAuthModePa
 }
 
 func (s *systemVolumesSuite) TestSystemVolumesActionReplacePlatformKeyAuthModePIN(c *C) {
+	if !secboot.WithSecbootSupport {
+		c.Skip("secboot is not available")
+	}
+
 	const authMode = device.AuthModePIN
 	s.testSystemVolumesActionReplacePlatformKey(c, authMode)
 }
@@ -633,6 +832,9 @@ func (s *systemVolumesSuite) TestSystemVolumesActionChangePassphrase(c *C) {
 		c.Check(authMode, Equals, device.AuthModePassphrase)
 		c.Check(keyslotRefs, DeepEquals, []fdestate.KeyslotRef{
 			{ContainerRole: "some-container-role", Name: "some-name"},
+			// keyslot expanded
+			{ContainerRole: "system-data", Name: "some-other-name"},
+			{ContainerRole: "system-save", Name: "some-other-name"},
 		})
 
 		return state.NewTaskSet(), nil
@@ -644,7 +846,8 @@ func (s *systemVolumesSuite) TestSystemVolumesActionChangePassphrase(c *C) {
 	"old-passphrase": "old",
 	"new-passphrase": "new",
 	"keyslots": [
-		{"container-role": "some-container-role", "name": "some-name"}
+		{"container-role": "some-container-role", "name": "some-name"},
+		{"name": "some-other-name"}
 	]
 }`)
 	req, err := http.NewRequest("POST", "/v2/system-volumes", body)
@@ -679,23 +882,6 @@ func (s *systemVolumesSuite) TestSystemVolumesActionChangePassphraseMissingOldPa
 	c.Assert(rsp.Message, Equals, "system volume action requires old-passphrase to be provided")
 }
 
-// This test should intentionally fail when resetting passphrase as root is implemented.
-func (s *systemVolumesSuite) TestSystemVolumesActionChangePassphraseMissingOldPassphraseAsRoot(c *C) {
-	s.daemon(c)
-	s.mockHybridSystem()
-
-	body := strings.NewReader(`{"action": "change-passphrase", "new-passphrase": "new"}`)
-	req, err := http.NewRequest("POST", "/v2/system-volumes", body)
-	c.Assert(err, IsNil)
-	req.Header.Add("Content-Type", "application/json")
-
-	s.asRootAuth(req)
-
-	rsp := s.errorReq(c, req, nil, actionIsExpected)
-	c.Assert(rsp.Status, Equals, 400)
-	c.Assert(rsp.Message, Equals, "system volume action requires old-passphrase to be provided")
-}
-
 func (s *systemVolumesSuite) TestSystemVolumesActionChangePassphraseMissingNewPassphrase(c *C) {
 	s.daemon(c)
 	s.mockHybridSystem()
@@ -760,12 +946,191 @@ func (s *systemVolumesSuite) TestSystemVolumesActionChangePassphraseChangeAuthEr
 	c.Check(rsp.Value, DeepEquals, []fdestate.KeyslotRef{{ContainerRole: "some-container-role", Name: "some-name"}})
 }
 
+func (s *systemVolumesSuite) TestSystemVolumesActionChangePIN(c *C) {
+	if !secboot.WithSecbootSupport {
+		// needed for PIN validation
+		c.Skip("secboot is not available")
+	}
+
+	d := s.daemon(c)
+	s.mockHybridSystem()
+	st := d.Overlord().State()
+
+	d.Overlord().Loop()
+	defer d.Overlord().Stop()
+
+	called := 0
+	s.AddCleanup(daemon.MockFdestateChangeAuth(func(st *state.State, authMode device.AuthMode, old, new string, keyslotRefs []fdestate.KeyslotRef) (*state.TaskSet, error) {
+		called++
+		c.Check(authMode, Equals, device.AuthModePIN)
+		c.Check(keyslotRefs, DeepEquals, []fdestate.KeyslotRef{
+			{ContainerRole: "some-container-role", Name: "some-name"},
+			// keyslot expanded
+			{ContainerRole: "system-data", Name: "some-other-name"},
+			{ContainerRole: "system-save", Name: "some-other-name"},
+		})
+
+		return state.NewTaskSet(), nil
+	}))
+
+	body := strings.NewReader(`
+{
+	"action": "change-pin",
+	"old-pin": "1234",
+	"new-pin": "4321",
+	"keyslots": [
+		{"container-role": "some-container-role", "name": "some-name"},
+		{"name": "some-other-name"}
+	]
+}`)
+	req, err := http.NewRequest("POST", "/v2/system-volumes", body)
+	c.Assert(err, IsNil)
+	req.Header.Add("Content-Type", "application/json")
+
+	rsp := s.asyncReq(c, req, nil, actionIsExpected)
+	c.Assert(rsp.Status, Equals, 202)
+
+	st.Lock()
+	chg := st.Change(rsp.Change)
+	st.Unlock()
+	c.Check(chg, NotNil)
+	c.Check(chg.ID(), Equals, "1")
+	c.Check(chg.Kind(), Equals, "fde-change-pin")
+	c.Check(called, Equals, 1)
+}
+
+func (s *systemVolumesSuite) TestSystemVolumesActionChangePINMissingOldPIN(c *C) {
+	s.daemon(c)
+	s.mockHybridSystem()
+
+	body := strings.NewReader(`{"action": "change-pin", "new-pin": "1234"}`)
+	req, err := http.NewRequest("POST", "/v2/system-volumes", body)
+	c.Assert(err, IsNil)
+	req.Header.Add("Content-Type", "application/json")
+
+	s.asUserAuth(c, req)
+
+	rsp := s.errorReq(c, req, nil, actionIsExpected)
+	c.Assert(rsp.Status, Equals, 400)
+	c.Assert(rsp.Message, Equals, "system volume action requires old-pin to be provided")
+}
+
+func (s *systemVolumesSuite) TestSystemVolumesActionChangePINInvalidOldPIN(c *C) {
+	if !secboot.WithSecbootSupport {
+		// needed for PIN validation
+		c.Skip("secboot is not available")
+	}
+
+	s.daemon(c)
+	s.mockHybridSystem()
+
+	body := strings.NewReader(`{"action": "change-pin", "old-pin": "old", "new-pin": "1234"}`)
+	req, err := http.NewRequest("POST", "/v2/system-volumes", body)
+	c.Assert(err, IsNil)
+	req.Header.Add("Content-Type", "application/json")
+
+	s.asUserAuth(c, req)
+
+	rsp := s.errorReq(c, req, nil, actionIsExpected)
+	c.Assert(rsp.Status, Equals, 400)
+	c.Assert(rsp.Message, Equals, "cannot change pin: invalid old-pin: invalid PIN: unexpected character")
+}
+
+func (s *systemVolumesSuite) TestSystemVolumesActionChangePINMissingNewPIN(c *C) {
+	s.daemon(c)
+	s.mockHybridSystem()
+
+	body := strings.NewReader(`{"action": "change-pin", "old-pin": "1234"}`)
+	req, err := http.NewRequest("POST", "/v2/system-volumes", body)
+	c.Assert(err, IsNil)
+	req.Header.Add("Content-Type", "application/json")
+
+	rsp := s.errorReq(c, req, nil, actionIsExpected)
+	c.Assert(rsp.Status, Equals, 400)
+	c.Assert(rsp.Message, Equals, "system volume action requires new-pin to be provided")
+}
+
+func (s *systemVolumesSuite) TestSystemVolumesActionChangePINInvalidNewPIN(c *C) {
+	if !secboot.WithSecbootSupport {
+		// needed for PIN validation
+		c.Skip("secboot is not available")
+	}
+
+	s.daemon(c)
+	s.mockHybridSystem()
+
+	body := strings.NewReader(`{"action": "change-pin", "old-pin": "1234", "new-pin": "new"}`)
+	req, err := http.NewRequest("POST", "/v2/system-volumes", body)
+	c.Assert(err, IsNil)
+	req.Header.Add("Content-Type", "application/json")
+
+	rsp := s.errorReq(c, req, nil, actionIsExpected)
+	c.Assert(rsp.Status, Equals, 400)
+	c.Assert(rsp.Message, Equals, "cannot change pin: invalid new-pin: invalid PIN: unexpected character")
+}
+
+func (s *systemVolumesSuite) TestSystemVolumesActionChangePINChangeAuthError(c *C) {
+	if !secboot.WithSecbootSupport {
+		// needed for PIN validation
+		c.Skip("secboot is not available")
+	}
+
+	s.daemon(c)
+	s.mockHybridSystem()
+
+	var mockErr error
+	s.AddCleanup(daemon.MockFdestateChangeAuth(func(st *state.State, authMode device.AuthMode, old, new string, keyslotRefs []fdestate.KeyslotRef) (*state.TaskSet, error) {
+		return nil, mockErr
+	}))
+
+	// catch all, bad request error
+	body := strings.NewReader(`{"action": "change-pin", "old-pin": "1234", "new-pin": "4321"}`)
+	req, err := http.NewRequest("POST", "/v2/system-volumes", body)
+	c.Assert(err, IsNil)
+	req.Header.Add("Content-Type", "application/json")
+
+	mockErr = errors.New("boom!")
+	rsp := s.errorReq(c, req, nil, actionIsExpected)
+	c.Assert(rsp.Status, Equals, 400)
+	c.Assert(rsp.Message, Equals, "cannot change pin: boom!")
+
+	// typed conflict detection error kind
+	mockErr = &snapstate.ChangeConflictError{
+		Message: fmt.Sprintf("conflict error: boom!"),
+	}
+	body = strings.NewReader(`{"action": "change-pin", "old-pin": "1234", "new-pin": "4321"}`)
+	req, err = http.NewRequest("POST", "/v2/system-volumes", body)
+	req.Header.Add("Content-Type", "application/json")
+
+	c.Assert(err, IsNil)
+	rsp = s.errorReq(c, req, nil, actionIsExpected)
+	c.Assert(rsp.Status, Equals, 409)
+	c.Check(rsp.Kind, Equals, client.ErrorKindSnapChangeConflict)
+	c.Check(rsp.Message, Equals, "conflict error: boom!")
+
+	// typed keyslots not found error kind
+	mockErr = &fdestate.KeyslotRefsNotFoundError{
+		KeyslotRefs: []fdestate.KeyslotRef{{ContainerRole: "some-container-role", Name: "some-name"}},
+	}
+	body = strings.NewReader(`{"action": "change-pin", "old-pin": "1234", "new-pin": "4321"}`)
+	req, err = http.NewRequest("POST", "/v2/system-volumes", body)
+	req.Header.Add("Content-Type", "application/json")
+
+	c.Assert(err, IsNil)
+	rsp = s.errorReq(c, req, nil, actionIsExpected)
+	c.Assert(rsp.Status, Equals, 400)
+	c.Check(rsp.Kind, Equals, client.ErrorKindKeyslotsNotFound)
+	c.Check(rsp.Message, Equals, `key slot reference (container-role: "some-container-role", name: "some-name") not found`)
+	c.Check(rsp.Value, DeepEquals, []fdestate.KeyslotRef{{ContainerRole: "some-container-role", Name: "some-name"}})
+}
+
 type mockKeyData struct {
 	authMode     device.AuthMode
 	platformName string
 	roles        []string
 
 	changePassphrase func(oldPassphrase, newPassphrase string) error
+	changePIN        func(oldPIN, newPIN string) error
 	writeTokenAtomic func(devicePath, slotName string) error
 }
 
@@ -788,6 +1153,13 @@ func (k *mockKeyData) ChangePassphrase(oldPassphrase, newPassphrase string) erro
 	return nil
 }
 
+func (k *mockKeyData) ChangePIN(oldPIN, newPIN string) error {
+	if k.changePIN != nil {
+		return k.changePIN(oldPIN, newPIN)
+	}
+	return nil
+}
+
 func (k *mockKeyData) WriteTokenAtomic(devicePath, slotName string) error {
 	if k.writeTokenAtomic != nil {
 		return k.writeTokenAtomic(devicePath, slotName)
@@ -964,7 +1336,7 @@ func (s *systemVolumesSuite) TestSystemVolumesActionCheckPassphrase(c *C) {
 	const expectedMinEntropy = uint32(20)
 	const expectedOptimalEntropy = uint32(50)
 
-	restore := daemon.MockDeviceValidatePassphrase(func(mode device.AuthMode, s string) (device.AuthQuality, error) {
+	restore := daemon.MockDeviceCheckAuthQuality(func(mode device.AuthMode, s string) (device.AuthQuality, error) {
 		c.Check(mode, Equals, device.AuthModePassphrase)
 		return device.AuthQuality{
 			Entropy:        expectedEntropy,
@@ -1032,7 +1404,7 @@ func (s *systemVolumesSuite) TestSystemVolumesActionCheckPassphraseError(c *C) {
 			"passphrase": tc.passphrase,
 		}
 
-		restore := daemon.MockDeviceValidatePassphrase(func(mode device.AuthMode, s string) (device.AuthQuality, error) {
+		restore := daemon.MockDeviceCheckAuthQuality(func(mode device.AuthMode, s string) (device.AuthQuality, error) {
 			c.Check(mode, Equals, device.AuthModePassphrase)
 			return device.AuthQuality{}, &device.AuthQualityError{
 				Reasons: []device.AuthQualityErrorReason{device.AuthQualityErrorReasonLowEntropy},
@@ -1069,7 +1441,7 @@ func (s *systemVolumesSuite) TestSystemVolumesActionCheckPIN(c *C) {
 	const expectedMinEntropy = uint32(20)
 	const expectedOptimalEntropy = uint32(50)
 
-	restore := daemon.MockDeviceValidatePassphrase(func(mode device.AuthMode, s string) (device.AuthQuality, error) {
+	restore := daemon.MockDeviceCheckAuthQuality(func(mode device.AuthMode, s string) (device.AuthQuality, error) {
 		c.Check(mode, Equals, device.AuthModePIN)
 		return device.AuthQuality{
 			Entropy:        expectedEntropy,
@@ -1137,7 +1509,7 @@ func (s *systemVolumesSuite) TestSystemVolumesActionCheckPINError(c *C) {
 			"pin":    tc.pin,
 		}
 
-		restore := daemon.MockDeviceValidatePassphrase(func(mode device.AuthMode, s string) (device.AuthQuality, error) {
+		restore := daemon.MockDeviceCheckAuthQuality(func(mode device.AuthMode, s string) (device.AuthQuality, error) {
 			c.Check(mode, Equals, device.AuthModePIN)
 			return device.AuthQuality{}, &device.AuthQualityError{
 				Reasons: []device.AuthQualityErrorReason{device.AuthQualityErrorReasonLowEntropy},
