@@ -170,6 +170,9 @@ func storageEncryption(encInfo *install.EncryptionSupportInfo) *client.StorageEn
 	if encInfo.PassphraseAuthAvailable {
 		storageEnc.Features = append(storageEnc.Features, client.StorageEncryptionFeaturePassphraseAuth)
 	}
+	if encInfo.PINAuthAvailable {
+		storageEnc.Features = append(storageEnc.Features, client.StorageEncryptionFeaturePINAuth)
+	}
 
 	return storageEnc
 }
@@ -177,6 +180,7 @@ func storageEncryption(encInfo *install.EncryptionSupportInfo) *client.StorageEn
 var (
 	devicestateInstallFinish                 = devicestate.InstallFinish
 	devicestateInstallSetupStorageEncryption = devicestate.InstallSetupStorageEncryption
+	devicestateInstallPreseed                = devicestate.InstallPreseed
 	devicestateCreateRecoverySystem          = devicestate.CreateRecoverySystem
 	devicestateRemoveRecoverySystem          = devicestate.RemoveRecoverySystem
 	devicestateGeneratePreInstallRecoveryKey = devicestate.GeneratePreInstallRecoveryKey
@@ -414,6 +418,19 @@ func postSystemActionInstall(c *Command, systemLabel string, req *systemActionRe
 		}
 		ensureStateSoon(st)
 		return AsyncResponse(nil, chg.ID())
+	case client.InstallStepPreseed:
+		if req.TargetRoot == nil {
+			return BadRequest("cannot preseed installed system without its target root")
+		}
+
+		chg, err := devicestateInstallPreseed(st, systemLabel, *req.TargetRoot)
+		if err != nil {
+			return InternalError("cannot preseed installed system: %v", err)
+		}
+
+		ensureStateSoon(st)
+		return AsyncResponse(nil, chg.ID())
+
 	default:
 		return BadRequest("unsupported install step %q", req.Step)
 	}
@@ -652,10 +669,10 @@ func postSystemActionRemove(c *Command, systemLabel string) Response {
 	return AsyncResponse(nil, chg.ID())
 }
 
-var deviceValidatePassphrase = device.ValidatePassphrase
+var deviceCheckAuthQuality = device.CheckAuthQuality
 
-func postValidatePassphrase(mode device.AuthMode, passphrase string) Response {
-	result, err := deviceValidatePassphrase(mode, passphrase)
+func postCheckAuthQuality(mode device.AuthMode, authVal string) Response {
+	result, err := deviceCheckAuthQuality(mode, authVal)
 	if err != nil {
 		var qualityErr *device.AuthQualityError
 		if errors.As(err, &qualityErr) {
@@ -712,7 +729,7 @@ func postSystemActionCheckPassphrase(c *Command, systemLabel string, req *system
 		}
 	}
 
-	return postValidatePassphrase(device.AuthModePassphrase, req.Passphrase)
+	return postCheckAuthQuality(device.AuthModePassphrase, req.Passphrase)
 }
 
 func postSystemActionCheckPIN(c *Command, systemLabel string, req *systemActionRequest) Response {
@@ -740,7 +757,7 @@ func postSystemActionCheckPIN(c *Command, systemLabel string, req *systemActionR
 		}
 	}
 
-	return postValidatePassphrase(device.AuthModePIN, req.PIN)
+	return postCheckAuthQuality(device.AuthModePIN, req.PIN)
 }
 
 func postSystemActionFixEncryptionSupport(c *Command, systemLabel string, req *systemActionRequest) Response {
