@@ -22,7 +22,9 @@ package daemon
 import (
 	"time"
 
+	"github.com/snapcore/snapd/overlord/fdestate"
 	"github.com/snapcore/snapd/overlord/state"
+	"github.com/snapcore/snapd/testutil"
 )
 
 func MockBuildID(mock string) (restore func()) {
@@ -64,3 +66,13 @@ func MockSnapstateSnapsAffectedByTask(f func(t *state.Task) ([]string, error)) (
 		snapstateSnapsAffectedByTask = old
 	}
 }
+
+func MockSnapdtoolsIsReexecd(f func() (bool, error)) (restore func()) {
+	return testutil.Mock(&snapdtoolIsReexecd, f)
+}
+
+func MockFdestateSystemState(f func(*state.State) (*fdestate.FDESystemState, error)) (restore func()) {
+	old := fdestateSystemState
+	fdestateSystemState = f
+	return func() { fdestateSystemState = old }
+}
