@@ -158,6 +158,15 @@ func (et EncryptionType) IsLUKS() bool {
 	return et == EncryptionTypeLUKS || et == EncryptionTypeLUKSWithICE
 }
 
+// ValidatePIN checks that the passed PIN is formatted properly.
+// If the supplied PIN larger than 255 characters or contains
+// anything other than base-10 digits, an error will be returned.
+var ValidatePIN func(pin string) error = validatePINImpl
+
+func validatePINImpl(pin string) error {
+	return errors.New("PIN validation callback not set up")
+}
+
 // AuthMode corresponds to an authentication mechanism.
 type AuthMode string
 
@@ -196,18 +205,20 @@ func (o *VolumesAuthOptions) Validate() error {
 		if len(o.PIN) == 0 {
 			return fmt.Errorf("pin cannot be empty")
 		}
+		if err := ValidatePIN(o.PIN); err != nil {
+			return err
+		}
 		if o.KDFType != "" {
 			return fmt.Errorf("%q authentication mode does not support custom kdf types", AuthModePIN)
 		}
-		return fmt.Errorf("%q authentication mode is not implemented", AuthModePIN)
 	default:
-		return fmt.Errorf("invalid authentication mode %q, only %q and %q modes are supported", o.Mode, AuthModePassphrase, AuthModePIN)
+		return fmt.Errorf("cannot use authentication mode %q, only %q and %q modes are supported", o.Mode, AuthModePassphrase, AuthModePIN)
 	}
 
 	switch o.KDFType {
 	case "argon2i", "argon2id", "pbkdf2", "":
 	default:
-		return fmt.Errorf("invalid kdf type %q, only \"argon2i\", \"argon2id\" and \"pbkdf2\" are supported", o.KDFType)
+		return fmt.Errorf("cannot use kdf type %q, only \"argon2i\", \"argon2id\" and \"pbkdf2\" are supported", o.KDFType)
 	}
 
 	if o.KDFTime != 0 {
@@ -255,11 +266,12 @@ type AuthQuality struct {
 // Hook setup by secboot to calculate entropy for PINs and passphrases.
 // PINs will be supplied as a numeric passphrase.
 //
-// Note: in most cases ValidatePassphrase should be used instead.
-var EntropyBits func(passphrase string) (uint32, error) = entropyBitsImpl
+// Note: in most cases CheckAuthQuality should be used instead
+// as it provides structured errors instead of raw entropy values.
+var EntropyBits func(authVal string) (uint32, error) = entropyBitsImpl
 
-func entropyBitsImpl(passphrase string) (uint32, error) {
-	panic("entropy bits calculation callback not set up")
+func entropyBitsImpl(authVal string) (uint32, error) {
+	return 0, errors.New("entropy bits calculation callback not set up")
 }
 
 const (
@@ -272,18 +284,18 @@ const (
 	optimalPINEntropyBits uint32 = 64
 )
 
-// ValidatePassphrase checks quality of given passphrase or PIN based
+// CheckAuthQuality checks quality of given passphrase or PIN based
 // on their entropy. An AuthQualityError error is returned which contains
 // more information about the given passphrase or PIN quality.
 //
 // PINs will be supplied as a numeric passphrase.
-func ValidatePassphrase(mode AuthMode, passphrase string) (AuthQuality, error) {
+func CheckAuthQuality(mode AuthMode, authVal string) (AuthQuality, error) {
 	minEntropy, optimalEntropy := minPassphraseEntropyBits, optimalPassphraseEntropyBits
 	if mode == AuthModePIN {
 		minEntropy, optimalEntropy = minPINEntropyBits, optimalPINEntropyBits
 	}
 
-	entropy, err := EntropyBits(passphrase)
+	entropy, err := EntropyBits(authVal)
 	if err != nil {
 		return AuthQuality{}, err
 	}
