@@ -162,7 +162,7 @@ func Prepare(opts *Options) error {
 			return fmt.Errorf("cannot read extra assertion: %s", err)
 		}
 		defer assertionsFile.Close()
-		extraAssertions, err := decodeExtraAssertions(assertionsFile)
+		extraAssertions, err := decodeExtraAssertions(assertionsFile, model.Grade())
 		if err != nil {
 			return err
 		}
@@ -1048,12 +1048,11 @@ var setupSeed = func(tsto *tooling.ToolingStore, model *asserts.Model, opts *Opt
 		return err
 	}
 
+	// Default validation behavior to "enforce" if not set
 	if opts.Customizations.Validation == "" {
-		if !opts.Classic {
-			fmt.Fprintf(Stderr, "WARNING: proceeding to download snaps ignoring validations, this default will change in the future. For now use --validation=enforce for validations to be taken into account, pass instead --validation=ignore to preserve current behavior going forward\n")
-		}
-		opts.Customizations.Validation = "ignore"
+		opts.Customizations.Validation = "enforce"
 	}
+	fmt.Fprintf(Stdout, "INFO: validation mode: %s\n", opts.Customizations.Validation)
 
 	assertMaxFormatsSelected := false
 	var assertMaxFormats map[string]int
@@ -1118,7 +1117,7 @@ var setupSeed = func(tsto *tooling.ToolingStore, model *asserts.Model, opts *Opt
 	return s.finish()
 }
 
-func decodeExtraAssertions(r io.Reader) ([]asserts.Assertion, error) {
+func decodeExtraAssertions(r io.Reader, grade asserts.ModelGrade) ([]asserts.Assertion, error) {
 	var extraAssertions []asserts.Assertion
 
 	dec := asserts.NewDecoder(r)
@@ -1134,6 +1133,14 @@ func decodeExtraAssertions(r io.Reader) ([]asserts.Assertion, error) {
 		switch a.Type() {
 		case asserts.SnapDeclarationType, asserts.SnapRevisionType, asserts.ModelType, asserts.SerialType, asserts.ValidationSetType:
 			return nil, fmt.Errorf("assertion type %v is not allowed for extra assertions", a.Type().Name)
+		case asserts.SystemUserType:
+			if grade != asserts.ModelDangerous {
+				return nil, fmt.Errorf("seeding system-user assertions is allowed for dangerous grade model only")
+			}
+			if a.HeaderString("password") != "" {
+				return nil, fmt.Errorf("seeded system-user assertions must not contain a password for security reasons, please use public key authentication instead")
+			}
+			fmt.Fprintf(Stderr, "INFO: the provided system-user assertion for user %s will be imported on first boot\n", a.HeaderString("username"))
 		}
 
 		extraAssertions = append(extraAssertions, a)
