@@ -115,6 +115,31 @@ func MockConnectedSlot(c *C, yaml string, si *snap.SideInfo, slotName string) (*
 	panic(fmt.Sprintf("cannot find slot %q in snap %q", slotName, info.InstanceName()))
 }
 
+type compRawInfo struct {
+	yaml string
+	rev  snap.Revision
+}
+
+// mockConnectedSlotWithComps mocks a connected slot for a snap with
+// components. Note that the components provided are expected to be defined in
+// the yaml of the snap too.
+func mockConnectedSlotWithComps(c *C, yaml string, si *snap.SideInfo, compsRaw []compRawInfo, slotName string) (*interfaces.ConnectedSlot, *snap.SlotInfo) {
+	info := snaptest.MockInfo(c, yaml, si)
+	compInfos := make([]*snap.ComponentInfo, 0, len(compsRaw))
+	for _, comp := range compsRaw {
+		compInfos = append(compInfos, snaptest.MockComponentInfo(
+			c, comp.yaml, snap.ComponentSideInfo{Revision: comp.rev}))
+	}
+
+	set, err := interfaces.NewSnapAppSet(info, compInfos)
+	c.Assert(err, IsNil)
+
+	if slotInfo, ok := info.Slots[slotName]; ok {
+		return interfaces.NewConnectedSlot(slotInfo, set, nil, nil), slotInfo
+	}
+	panic(fmt.Sprintf("cannot find slot %q in snap %q", slotName, info.InstanceName()))
+}
+
 func MockHotplugSlot(c *C, yaml string, si *snap.SideInfo, hotplugKey snap.HotplugKey, ifaceName, slotName string, staticAttrs map[string]any) *snap.SlotInfo {
 	info := snaptest.MockInfo(c, yaml, si)
 	if _, ok := info.Slots[slotName]; ok {
@@ -261,13 +286,14 @@ func (s *desktopFileRulesBaseSuite) TestDesktopFileRulesHappy(c *C) {
 	opts := testDesktopFileRulesOptions{
 		snapName:       "some-snap",
 		desktopFiles:   []string{"org.example.desktop", "org.example.Foo.desktop", "org.example.Bar.desktop", "bar.desktop"},
-		desktopFileIDs: []string{"org.example", "org.example.Foo"},
+		desktopFileIDs: []string{"org.example.desktop", "org.example.Foo.desktop", "org.example.Foo.WithoutSuffix"},
 		isInstance:     false,
 		expectedRules: []string{
 			// allow rules for snap's desktop files
-			fmt.Sprintf("%s/@{SNAP_INSTANCE_DESKTOP}_*.desktop r,", dirs.SnapDesktopFilesDir),        // prefixed with DesktopPrefix()
-			fmt.Sprintf("%s r,", filepath.Join(dirs.SnapDesktopFilesDir, "org.example.desktop")),     // desktop-file-ids, unchanged
-			fmt.Sprintf("%s r,", filepath.Join(dirs.SnapDesktopFilesDir, "org.example.Foo.desktop")), // desktop-file-ids, unchanged
+			fmt.Sprintf("%s/@{SNAP_INSTANCE_DESKTOP}_*.desktop r,", dirs.SnapDesktopFilesDir),                      // prefixed with DesktopPrefix()
+			fmt.Sprintf("%s r,", filepath.Join(dirs.SnapDesktopFilesDir, "org.example.desktop")),                   // desktop-file-ids, unchanged
+			fmt.Sprintf("%s r,", filepath.Join(dirs.SnapDesktopFilesDir, "org.example.Foo.desktop")),               // desktop-file-ids, unchanged
+			fmt.Sprintf("%s r,", filepath.Join(dirs.SnapDesktopFilesDir, "org.example.Foo.WithoutSuffix.desktop")), // desktop-file-ids, unchanged
 			// check all deny patterns are generated
 			fmt.Sprintf("deny %s r,", filepath.Join(dirs.SnapDesktopFilesDir, "[^so]**.desktop")), // ^s from some-snap and ^o from org
 			fmt.Sprintf("deny %s r,", filepath.Join(dirs.SnapDesktopFilesDir, "{o[^r],s[^o]}**.desktop")),
@@ -425,7 +451,7 @@ func (s *desktopFileRulesBaseSuite) TestDesktopFileRulesBadDesktopFileIDs(c *C)
 		desktopFiles:   []string{"org.*.example.desktop"},
 		desktopFileIDs: []string{"org.*.example"},
 		expectedRules:  s.fallbackRules,
-		expectedErr:    `internal error: invalid desktop file ID "org.*.example" found in snap "some-snap": "org.*.example" contains a reserved apparmor char from ` + "?*[]{}^\"\x00",
+		expectedErr:    `internal error: invalid desktop file ID "org.*.example.desktop" found in snap "some-snap": "org.*.example.desktop" contains a reserved apparmor char from ` + "?*[]{}^\"\x00",
 	}
 	s.testDesktopFileRules(c, opts)
 }
