@@ -143,26 +143,27 @@ func (s *baseDeclSuite) TestAutoConnection(c *C) {
 	// these have more complex or in flux policies and have their
 	// own separate tests
 	snowflakes := map[string]bool{
-		"content":                true,
-		"core-support":           true,
-		"cuda-driver-libs":       true,
-		"desktop":                true,
-		"egl-driver-libs":        true,
-		"gbm-driver-libs":        true,
-		"vulkan-driver-libs":     true,
-		"home":                   true,
-		"lxd-support":            true,
-		"microstack-support":     true,
-		"multipass-support":      true,
-		"opengl-driver-libs":     true,
-		"opengles-driver-libs":   true,
-		"packagekit-control":     true,
-		"pkcs11":                 true,
-		"remoteproc":             true,
-		"screen-inhibit-control": true,
-		"snapd-control":          true,
-		"upower-observe":         true,
-		"empty":                  true,
+		"content":                  true,
+		"core-support":             true,
+		"cuda-driver-libs":         true,
+		"desktop":                  true,
+		"egl-driver-libs":          true,
+		"gbm-driver-libs":          true,
+		"vulkan-driver-libs":       true,
+		"home":                     true,
+		"lxd-support":              true,
+		"microstack-support":       true,
+		"multipass-support":        true,
+		"nvidia-video-driver-libs": true,
+		"opengl-driver-libs":       true,
+		"opengles-driver-libs":     true,
+		"packagekit-control":       true,
+		"pkcs11":                   true,
+		"remoteproc":               true,
+		"screen-inhibit-control":   true,
+		"snapd-control":            true,
+		"upower-observe":           true,
+		"empty":                    true,
 	}
 
 	// these simply auto-connect, anything else doesn't
@@ -882,23 +883,24 @@ var (
 		"wayland":                   {"app", "core"},
 		"x11":                       {"app", "core"},
 		// snowflakes
-		"classic-support":      nil,
-		"cuda-driver-libs":     nil,
-		"custom-device":        nil,
-		"docker":               nil,
-		"egl-driver-libs":      nil,
-		"gbm-driver-libs":      nil,
-		"vulkan-driver-libs":   nil,
-		"lxd":                  nil,
-		"microceph":            nil,
-		"microceph-support":    nil,
-		"microovn":             nil,
-		"opengl-driver-libs":   nil,
-		"opengles-driver-libs": nil,
-		"pkcs11":               nil,
-		"posix-mq":             nil,
-		"shared-memory":        nil,
-		"gpio-chardev":         nil,
+		"classic-support":          nil,
+		"cuda-driver-libs":         nil,
+		"custom-device":            nil,
+		"docker":                   nil,
+		"egl-driver-libs":          nil,
+		"gbm-driver-libs":          nil,
+		"vulkan-driver-libs":       nil,
+		"lxd":                      nil,
+		"microceph":                nil,
+		"microceph-support":        nil,
+		"microovn":                 nil,
+		"nvidia-video-driver-libs": nil,
+		"opengl-driver-libs":       nil,
+		"opengles-driver-libs":     nil,
+		"pkcs11":                   nil,
+		"posix-mq":                 nil,
+		"shared-memory":            nil,
+		"gpio-chardev":             nil,
 	}
 
 	restrictedPlugInstallation = map[string][]string{
@@ -1077,6 +1079,7 @@ func (s *baseDeclSuite) TestPlugInstallation(c *C) {
 		"nomad-support":                    true,
 		"nvidia-drivers-support":           true,
 		"nvme-control":                     true,
+		"nvidia-video-driver-libs":         true,
 		"opengl-driver-libs":               true,
 		"opengles-driver-libs":             true,
 		"packagekit-control":               true,
@@ -1393,6 +1396,7 @@ func (s *baseDeclSuite) TestValidity(c *C) {
 		"nomad-support":                    true,
 		"nvidia-drivers-support":           true,
 		"nvme-control":                     true,
+		"nvidia-video-driver-libs":         true,
 		"opengl-driver-libs":               true,
 		"opengles-driver-libs":             true,
 		"packagekit-control":               true,
