@@ -150,10 +150,11 @@ func (s *deviceMgrBootconfigSuite) setupClassicWithModesModel(c *C) *asserts.Mod
 }
 
 type testBootConfigUpdateOpts struct {
-	updateAttempted     bool
-	updateApplied       bool
-	cmdlineAppend       string
-	cmdlineAppendDanger string
+	updateAttempted                  bool
+	updateApplied                    bool
+	cmdlineAppend                    string
+	cmdlineAppendDanger              string
+	extraSnapdKernelCmdlineFragments map[string]string
 }
 
 func (s *deviceMgrBootconfigSuite) testBootConfigUpdateRun(c *C, opts testBootConfigUpdateOpts, errMatch string) {
@@ -181,6 +182,18 @@ kernel-cmdline:
 	c.Assert(err, IsNil)
 	tr.Commit()
 
+	// Set extra snapd kernel command line args as well
+	for fragmentID, fragment := range opts.extraSnapdKernelCmdlineFragments {
+		updated, err := devicestate.SetExtraSnapdKernelCommandLineFragment(s.state, devicestate.ExtraSnapdKernelCmdlineFragmentID(fragmentID), fragment)
+		c.Assert(err, IsNil)
+		c.Check(updated, Equals, true)
+	}
+	if len(opts.extraSnapdKernelCmdlineFragments) == 0 {
+		checkPendingExtraSnapdFragments(c, s.state, false)
+	} else {
+		checkPendingExtraSnapdFragments(c, s.state, true)
+	}
+
 	tsk := s.state.NewTask("update-managed-boot-config", "update boot config")
 	tsk.Set("snap-setup", &snapstate.SnapSetup{
 		SideInfo: &s.gadgetSnapInfo.SideInfo,
@@ -262,6 +275,18 @@ kernel-cmdline:
 	c.Assert(err, IsNil)
 	tr.Commit()
 
+	// Set extra snapd kernel command line args as well
+	for fragmentID, fragment := range opts.extraSnapdKernelCmdlineFragments {
+		updated, err := devicestate.SetExtraSnapdKernelCommandLineFragment(s.state, devicestate.ExtraSnapdKernelCmdlineFragmentID(fragmentID), fragment)
+		c.Assert(err, IsNil)
+		c.Check(updated, Equals, true)
+	}
+	if len(opts.extraSnapdKernelCmdlineFragments) == 0 {
+		checkPendingExtraSnapdFragments(c, s.state, false)
+	} else {
+		checkPendingExtraSnapdFragments(c, s.state, true)
+	}
+
 	tsk := s.state.NewTask("update-managed-boot-config", "update boot config")
 	tsk.Set("snap-setup", &snapstate.SnapSetup{
 		SideInfo: &s.gadgetSnapInfo.SideInfo,
@@ -375,6 +400,61 @@ func (s *deviceMgrBootconfigSuite) TestBootConfigUpdateRunSuccessClassicWithMode
 	})
 }
 
+func (s *deviceMgrBootconfigSuite) TestBootConfigUpdateRunSuccessClassicWithModesWithExtraSnapdArgs(c *C) {
+	s.state.Lock()
+	s.setupClassicWithModesModel(c)
+	s.state.Unlock()
+
+	s.managedbl.Updated = true
+
+	opts := testBootConfigUpdateOpts{
+		updateAttempted: true,
+		updateApplied:   true,
+		extraSnapdKernelCmdlineFragments: map[string]string{
+			"xkb": `arg1="val-1" arg1="val-2" arg2`,
+		},
+	}
+	s.testBootConfigUpdateRunClassic(c, opts, "")
+
+	m, err := boot.ReadModeenv("")
+	c.Assert(err, IsNil)
+	c.Check([]string(m.CurrentKernelCommandLines), DeepEquals, []string{
+		"snapd_recovery_mode=run console=ttyS0 console=tty1 panic=-1",
+		`snapd_recovery_mode=run console=ttyS0 console=tty1 panic=-1 candidate arg1="val-1" arg1="val-2" arg2`,
+	})
+	s.state.Lock()
+	checkPendingExtraSnapdFragments(c, s.state, false)
+	s.state.Unlock()
+}
+
+func (s *deviceMgrBootconfigSuite) TestBootConfigUpdateRunSuccessClassicWithModesWithExtraSnapdArgsAndAppend(c *C) {
+	s.state.Lock()
+	s.setupClassicWithModesModel(c)
+	s.state.Unlock()
+
+	s.managedbl.Updated = true
+
+	opts := testBootConfigUpdateOpts{
+		updateAttempted: true,
+		updateApplied:   true,
+		cmdlineAppend:   "par1=val par2",
+		extraSnapdKernelCmdlineFragments: map[string]string{
+			"xkb": `snapd.xkb="some-value"`,
+		},
+	}
+	s.testBootConfigUpdateRunClassic(c, opts, "")
+
+	m, err := boot.ReadModeenv("")
+	c.Assert(err, IsNil)
+	c.Check([]string(m.CurrentKernelCommandLines), DeepEquals, []string{
+		"snapd_recovery_mode=run console=ttyS0 console=tty1 panic=-1",
+		`snapd_recovery_mode=run console=ttyS0 console=tty1 panic=-1 candidate snapd.xkb="some-value" par1=val par2`,
+	})
+	s.state.Lock()
+	checkPendingExtraSnapdFragments(c, s.state, false)
+	s.state.Unlock()
+}
+
 func (s *deviceMgrBootconfigSuite) TestBootConfigUpdateWithGadgetExtra(c *C) {
 	s.state.Lock()
 	s.setupUC20Model(c)
@@ -510,6 +590,62 @@ func (s *deviceMgrBootconfigSuite) TestBootConfigUpdateRunWithAppendBothOptsSign
 	s.testBootConfigUpdateRunWithAppendBothOpts(c, "signed")
 }
 
+func (s *deviceMgrBootconfigSuite) TestBootConfigUpdateRunWithExtraSnapdArgs(c *C) {
+	s.state.Lock()
+	s.setupUC20Model(c)
+	s.state.Unlock()
+
+	s.managedbl.Updated = true
+
+	opts := testBootConfigUpdateOpts{
+		updateAttempted: true,
+		updateApplied:   true,
+		extraSnapdKernelCmdlineFragments: map[string]string{
+			"xkb": "some-value",
+		},
+	}
+	s.testBootConfigUpdateRun(c, opts, "")
+
+	m, err := boot.ReadModeenv("")
+	c.Assert(err, IsNil)
+	c.Check([]string(m.CurrentKernelCommandLines), DeepEquals, []string{
+		"snapd_recovery_mode=run console=ttyS0 console=tty1 panic=-1",
+		"snapd_recovery_mode=run console=ttyS0 console=tty1 panic=-1 candidate some-value",
+	})
+	s.state.Lock()
+	checkPendingExtraSnapdFragments(c, s.state, false)
+	s.state.Unlock()
+}
+
+func (s *deviceMgrBootconfigSuite) TestBootConfigUpdateRunWithExtraSnapdArgsAndAppend(c *C) {
+	s.state.Lock()
+	s.setupUC20Model(c)
+	s.state.Unlock()
+
+	s.managedbl.Updated = true
+
+	opts := testBootConfigUpdateOpts{
+		updateAttempted:     true,
+		updateApplied:       true,
+		cmdlineAppend:       "par1=val par2",
+		cmdlineAppendDanger: "par3=val par4",
+		extraSnapdKernelCmdlineFragments: map[string]string{
+			"xkb": "xkb-val",
+		},
+	}
+	s.testBootConfigUpdateRun(c, opts, "")
+
+	m, err := boot.ReadModeenv("")
+	c.Assert(err, IsNil)
+	c.Check([]string(m.CurrentKernelCommandLines), DeepEquals, []string{
+		"snapd_recovery_mode=run console=ttyS0 console=tty1 panic=-1",
+		`snapd_recovery_mode=run console=ttyS0 console=tty1 panic=-1 candidate xkb-val par1=val par2 par3=val par4`,
+	})
+	s.state.Lock()
+	checkPendingExtraSnapdFragments(c, s.state, false)
+	s.state.Unlock()
+}
+
 func (s *deviceMgrBootconfigSuite) TestBootConfigUpdateRunButNotUpdated(c *C) {
 	s.state.Lock()
 	s.setupUC20Model(c)
