@@ -33,6 +33,7 @@ import (
 	"github.com/snapcore/snapd/httputil"
 	"github.com/snapcore/snapd/kernel/fde"
 	"github.com/snapcore/snapd/osutil"
+	"github.com/snapcore/snapd/osutil/keyboard"
 	"github.com/snapcore/snapd/osutil/user"
 	"github.com/snapcore/snapd/overlord/fdestate"
 	"github.com/snapcore/snapd/overlord/snapstate"
@@ -286,15 +287,20 @@ func SetPostFactoryResetRan(m *DeviceManager, b bool) {
 	m.ensurePostFactoryResetRan = b
 }
 
+func SetEarlyBootLocaleConfigUpdatedRan(m *DeviceManager, b bool) {
+	m.ensureEarlyBootLocaleConfigUpdatedRan = b
+}
+
 func StartTime() time.Time {
 	return startTime
 }
 
 type (
-	RegistrationContext = registrationContext
-	RemodelContext      = remodelContext
-	SeededSystem        = seededSystem
-	RecoverySystemSetup = recoverySystemSetup
+	RegistrationContext               = registrationContext
+	RemodelContext                    = remodelContext
+	SeededSystem                      = seededSystem
+	RecoverySystemSetup               = recoverySystemSetup
+	ExtraSnapdKernelCmdlineFragmentID = extraSnapdKernelCommandLineFragmentID
 )
 
 func RegistrationCtx(m *DeviceManager, t *state.Task) (registrationContext, error) {
@@ -341,6 +347,8 @@ var (
 	LogNewSystemSnapFile                   = logNewSystemSnapFile
 	PurgeNewSystemSnapFiles                = purgeNewSystemSnapFiles
 	CreateRecoverySystemTasks              = createRecoverySystemTasks
+
+	SetExtraSnapdKernelCommandLineFragment = setExtraSnapdKernelCommandLineFragment
 )
 
 func MockApplyPreseededData(f func(deviceSeed seed.PreseedCapable, writableDir string) error) (restore func()) {
@@ -577,6 +585,18 @@ func EnsureExpiredUsersRemoved(m *DeviceManager) error {
 	return m.ensureExpiredUsersRemoved()
 }
 
+func MockKeyboardCurrentXKBConfig(f func() (*keyboard.XKBConfig, error)) (restore func()) {
+	return testutil.Mock(&keyboardCurrentXKBConfig, f)
+}
+
+func MockKeyboardNewXKBConfigListener(f func(ctx context.Context, cb func(config *keyboard.XKBConfig)) (*keyboard.XKBConfigListener, error)) (restore func()) {
+	return testutil.Mock(&keyboardNewXKBConfigListener, f)
+}
+
+func EnsureEarlyBootXKBConfigUpdated(m *DeviceManager) error {
+	return m.ensureEarlyBootXKBConfigUpdated()
+}
+
 var ProcessAutoImportAssertions = processAutoImportAssertions
 
 func MockCreateAllKnownSystemUsers(createAllUsers func(state *state.State, assertDb asserts.RODatabase, model *asserts.Model, serial *asserts.Serial, sudoer bool) ([]*CreatedUser, error)) (restore func()) {
@@ -588,7 +608,7 @@ func MockCreateAllKnownSystemUsers(createAllUsers func(state *state.State, asser
 func MockEncryptionSetupDataInCache(
 	st *state.State,
 	label string,
-	recoveryKeyID string,
+	rkey *keys.RecoveryKey,
 	volumesAuth *device.VolumesAuthOptions,
 	checkContext *secboot.PreinstallCheckContext,
 ) (restore func()) {
@@ -605,7 +625,7 @@ func MockEncryptionSetupDataInCache(
 			EncryptedDevice: "/dev/mapper/ubuntu-data",
 		},
 	}
-	esd = install.MockEncryptionSetupData(labelToEncData, recoveryKeyID, volumesAuth, checkContext)
+	esd = install.MockEncryptionSetupData(labelToEncData, rkey, volumesAuth, checkContext)
 	st.Cache(encryptionSetupDataKey{label}, esd)
 	return func() { CleanUpEncryptionSetupDataInCache(st, label) }
 }
