@@ -112,7 +112,10 @@ func (s *fdeMgrSuite) mockCurrentKeys(c *C, rkeys, unlockKeys []fdestate.Keyslot
 func (s *fdeMgrSuite) TestDoAddRecoveryKeys(c *C) {
 	const onClassic = true
 	manager := s.startedManager(c, onClassic)
-	s.mockCurrentKeys(c, nil, nil)
+	s.mockCurrentKeys(c,
+		[]fdestate.KeyslotRef{{ContainerRole: "system-data", Name: "some-key-1"}},
+		[]fdestate.KeyslotRef{{ContainerRole: "system-data", Name: "some-key-2"}},
+	)
 
 	s.st.Lock()
 	defer s.st.Unlock()
@@ -132,7 +135,7 @@ func (s *fdeMgrSuite) TestDoAddRecoveryKeys(c *C) {
 		},
 		{
 			keyslots: []fdestate.KeyslotRef{
-				{ContainerRole: "system-data", Name: "default-recovery"}, // already exists
+				{ContainerRole: "system-data", Name: "some-key-1"}, // already exists
 				{ContainerRole: "system-data", Name: "tmp-default-recovery"},
 			},
 			expectedAdds: []string{"/dev/disk/by-uuid/data:tmp-default-recovery"},
@@ -181,6 +184,18 @@ func (s *fdeMgrSuite) TestDoAddRecoveryKeys(c *C) {
 			expectedDeletes: []string{"/dev/disk/by-uuid/data:tmp-default-recovery"},
 			expectedErr:     `cannot add recovery key slot \(container-role: "system-save", name: "tmp-default-recovery"\): add error on /dev/disk/by-uuid/save:tmp-default-recovery`,
 		},
+		{
+			keyslots: []fdestate.KeyslotRef{
+				{ContainerRole: "system-data", Name: "some-key-3"},
+				{ContainerRole: "system-data", Name: "some-key-4"},
+				{ContainerRole: "system-data", Name: "some-key-5"},
+				{ContainerRole: "system-data", Name: "some-key-6"},
+				{ContainerRole: "system-data", Name: "some-key-7"},
+				{ContainerRole: "system-data", Name: "some-key-8"},
+				{ContainerRole: "system-data", Name: "some-key-9"},
+			},
+			expectedErr: `insufficient capacity on container system-data`,
+		},
 	}
 
 	for i, tc := range tcs {
@@ -609,7 +624,8 @@ func (s *fdeMgrSuite) TestDoRenameKeysMissingMapping(c *C) {
 
 func (s *fdeMgrSuite) TestDoRenameKeysRenameError(c *C) {
 	const onClassic = true
-	s.startedManager(c, onClassic)
+	m := s.startedManager(c, onClassic)
+	c.Assert(m.IsFunctional(), IsNil)
 	s.mockCurrentKeys(c, nil, nil)
 
 	defer fdestate.MockSecbootRenameContainerKey(func(devicePath, oldName, newName string) error {
@@ -690,6 +706,19 @@ func (s *fdeMgrSuite) TestDoChangeAuthKeys(c *C) {
 			},
 			authMode: device.AuthModePassphrase,
 		},
+		{
+			keyslots: []fdestate.KeyslotRef{
+				{ContainerRole: "system-data", Name: "default"},
+				{ContainerRole: "system-data", Name: "default-fallback"},
+				{ContainerRole: "system-save", Name: "default-fallback"},
+			},
+			expectedChanges: []string{
+				"/dev/disk/by-uuid/data:default",
+				"/dev/disk/by-uuid/data:default-fallback",
+				"/dev/disk/by-uuid/save:default-fallback",
+			},
+			authMode: device.AuthModePIN,
+		},
 		{
 			keyslots:    []fdestate.KeyslotRef{{ContainerRole: "system-data", Name: "default"}},
 			noOpt:       true,
@@ -718,14 +747,15 @@ func (s *fdeMgrSuite) TestDoChangeAuthKeys(c *C) {
 		},
 		{
 			keyslots:    []fdestate.KeyslotRef{{ContainerRole: "system-data", Name: "default"}},
-			authMode:    device.AuthModePassphrase,
-			errOn:       []string{"write:/dev/disk/by-uuid/data:default"},
-			expectedErr: `cannot write key data for \(container-role: "system-data", name: "default"\): write error on /dev/disk/by-uuid/data:default`,
+			authMode:    device.AuthModePIN,
+			errOn:       []string{"change:/dev/disk/by-uuid/data:default"},
+			expectedErr: `cannot change PIN for \(container-role: "system-data", name: "default"\): change error on /dev/disk/by-uuid/data:default`,
 		},
 		{
 			keyslots:    []fdestate.KeyslotRef{{ContainerRole: "system-data", Name: "default"}},
-			authMode:    device.AuthModePIN,
-			expectedErr: "internal error: changing PINs is not implemented",
+			authMode:    device.AuthModePassphrase,
+			errOn:       []string{"write:/dev/disk/by-uuid/data:default"},
+			expectedErr: `cannot write key data for \(container-role: "system-data", name: "default"\): write error on /dev/disk/by-uuid/data:default`,
 		},
 		{
 			keyslots:    []fdestate.KeyslotRef{{ContainerRole: "system-data", Name: "default"}},
@@ -738,9 +768,13 @@ func (s *fdeMgrSuite) TestDoChangeAuthKeys(c *C) {
 		task.Set("keyslots", tc.keyslots)
 		task.Set("auth-mode", tc.authMode)
 
+		old, new := "old", "new"
+		if tc.authMode == device.AuthModePIN {
+			old, new = "1234", "4321"
+		}
 		if !tc.noOpt {
 			s.st.Unlock()
-			defer fdestate.MockChangeAuthOptionsInCache(s.st, "old", "new")()
+			defer fdestate.MockChangeAuthOptionsInCache(s.st, old, new)()
 			s.st.Lock()
 		}
 
@@ -751,23 +785,25 @@ func (s *fdeMgrSuite) TestDoChangeAuthKeys(c *C) {
 			if strutil.ListContains(tc.errOn, fmt.Sprintf("read:%s:%s", devicePath, slotName)) {
 				return nil, fmt.Errorf("read error on %s", entry)
 			}
+			changeHandler := func(oldVal, newVal string) error {
+				switch changeCalls[entry] {
+				case 0:
+					c.Check(oldVal, Equals, old)
+					c.Check(newVal, Equals, new)
+				case 1:
+					c.Check(oldVal, Equals, new)
+					c.Check(newVal, Equals, old)
+				default:
+					panic("unexpected number of change calls")
+				}
+				if strutil.ListContains(tc.errOn, fmt.Sprintf("change:%s", entry)) {
+					return fmt.Errorf("change error on %s", entry)
+				}
+				return nil
+			}
 			return &mockKeyData{
-				changePassphrase: func(oldPassphrase, newPassphrase string) error {
-					switch changeCalls[entry] {
-					case 0:
-						c.Check(oldPassphrase, Equals, "old")
-						c.Check(newPassphrase, Equals, "new")
-					case 1:
-						c.Check(oldPassphrase, Equals, "new")
-						c.Check(newPassphrase, Equals, "old")
-					default:
-						panic("unexpected number of change calls")
-					}
-					if strutil.ListContains(tc.errOn, fmt.Sprintf("change:%s", entry)) {
-						return fmt.Errorf("change error on %s", entry)
-					}
-					return nil
-				},
+				changePassphrase: changeHandler,
+				changePIN:        changeHandler,
 				writeTokenAtomic: func(devicePath, slotName string) error {
 					if strutil.ListContains(tc.errOn, fmt.Sprintf("write:%s", entry)) {
 						return fmt.Errorf("write error on %s", entry)
@@ -816,8 +852,8 @@ func (s *fdeMgrSuite) TestDoChangeAuthKeys(c *C) {
 			if !tc.noOpt {
 				// Auth options are kept to account for re-runs
 				opts := fdestate.GetChangeAuthOptionsFromCache(s.st)
-				c.Assert(opts.New(), Equals, "new")
-				c.Assert(opts.Old(), Equals, "old")
+				c.Assert(opts.New(), Equals, new)
+				c.Assert(opts.Old(), Equals, old)
 			}
 		}
 	}
@@ -869,6 +905,7 @@ func (s *fdeMgrSuite) TestDoAddPlatformKeys(c *C) {
 		keyslots                      []fdestate.KeyslotRef
 		authMode                      device.AuthMode
 		noVolumesAuth                 bool
+		badAuthValue                  bool
 		recoveryMode                  bool
 		noop                          bool
 		roles                         [][]string
@@ -886,8 +923,16 @@ func (s *fdeMgrSuite) TestDoAddPlatformKeys(c *C) {
 			expectedAdds: []string{"/dev/disk/by-uuid/data:tmp-default"},
 		},
 		{
-			authMode:    device.AuthModePIN,
-			expectedErr: `internal error: invalid authentication options: "pin" authentication mode is not implemented`,
+			authMode:     device.AuthModePIN,
+			expectedAdds: []string{"/dev/disk/by-uuid/data:tmp-default"},
+		},
+		{
+			authMode: device.AuthModePassphrase, badAuthValue: true,
+			expectedErr: "internal error: invalid authentication options: passphrase cannot be empty",
+		},
+		{
+			authMode: device.AuthModePIN, badAuthValue: true,
+			expectedErr: "internal error: invalid authentication options: pin cannot be empty",
 		},
 		{
 			authMode: device.AuthModePIN, noVolumesAuth: true,
@@ -974,6 +1019,10 @@ func (s *fdeMgrSuite) TestDoAddPlatformKeys(c *C) {
 				volumesAuth.PIN = "1234"
 			}
 			s.st.Cache(fdestate.VolumesAuthOptionsKey(), volumesAuth)
+			if tc.badAuthValue {
+				volumesAuth.Passphrase = ""
+				volumesAuth.PIN = ""
+			}
 		}
 
 		defer fdestate.MockSecbootAddContainerTPMProtectedKey(func(devicePath string, slotName string, params *secboot.ProtectKeyParams) error {
