@@ -431,7 +431,7 @@ func (s *confdbSuite) TestConfdbSetSingleView(c *C) {
 	c.Assert(s.mockContext.Done(), IsNil)
 	s.mockContext.Unlock()
 
-	val, err := tx.Get(parsePath(c, "wifi.ssid"))
+	val, err := tx.Get(parsePath(c, "wifi.ssid"), nil)
 	c.Assert(err, IsNil)
 	c.Assert(val, DeepEquals, "other-ssid")
 }
@@ -460,7 +460,7 @@ func (s *confdbSuite) TestConfdbSetSingleViewNewTransaction(c *C) {
 
 	c.Assert(called, Equals, true)
 
-	val, err := tx.Get(parsePath(c, "wifi.ssid"))
+	val, err := tx.Get(parsePath(c, "wifi.ssid"), nil)
 	c.Assert(err, IsNil)
 	c.Assert(val, DeepEquals, "other-ssid")
 }
@@ -481,11 +481,11 @@ func (s *confdbSuite) TestConfdbSetManyViews(c *C) {
 	c.Check(stdout, IsNil)
 	c.Check(stderr, IsNil)
 
-	val, err := tx.Get(parsePath(c, "wifi.ssid"))
+	val, err := tx.Get(parsePath(c, "wifi.ssid"), nil)
 	c.Assert(err, IsNil)
 	c.Assert(val, Equals, "other-ssid")
 
-	val, err = tx.Get(parsePath(c, "wifi.psk"))
+	val, err = tx.Get(parsePath(c, "wifi.psk"), nil)
 	c.Assert(err, IsNil)
 	c.Assert(val, Equals, "other-secret")
 }
@@ -537,10 +537,10 @@ func (s *confdbSuite) TestConfdbSetExclamationMark(c *C) {
 	c.Check(stdout, IsNil)
 	c.Check(stderr, IsNil)
 
-	_, err = tx.Get(parsePath(c, "wifi.psk"))
+	_, err = tx.Get(parsePath(c, "wifi.psk"), nil)
 	c.Assert(err, testutil.ErrorIs, &confdb.NoDataError{})
 
-	val, err := tx.Get(parsePath(c, "wifi.ssid"))
+	val, err := tx.Get(parsePath(c, "wifi.ssid"), nil)
 	c.Assert(err, IsNil)
 	c.Assert(val, Equals, "foo")
 }
