@@ -89,6 +89,12 @@ type EncryptionSupportInfo struct {
 	// availability errors identified during preinstall check.
 	AvailabilityCheckErrors []secboot.PreinstallErrorDetails
 
+	// availabilityCheckContext holds the configuration and state captured during
+	// the preinstall check. It is required for performing follow-up checks with
+	// actions to resolve identified errors. It is also an indicator that the
+	// preinstall check was used instead of the general availability check.
+	availabilityCheckContext *secboot.PreinstallCheckContext
+
 	// PassphraseAuthAvailable is set if the passphrase authentication
 	// is supported.
 	PassphraseAuthAvailable bool
@@ -97,6 +103,11 @@ type EncryptionSupportInfo struct {
 	PINAuthAvailable bool
 }
 
+// CheckContext returns the underlying preinstall check context.
+func (esi *EncryptionSupportInfo) CheckContext() *secboot.PreinstallCheckContext {
+	return esi.availabilityCheckContext
+}
+
 // ComponentSeedInfo contains information for a component from the seed and
 // from its metadata.
 type ComponentSeedInfo struct {
@@ -124,6 +135,10 @@ var (
 
 	secbootCheckTPMKeySealingSupported = secboot.CheckTPMKeySealingSupported
 	secbootPreinstallCheck             = secboot.PreinstallCheck
+	secbootPreinstallCheckAction       = (*secboot.PreinstallCheckContext).PreinstallCheckAction
+	secbootSaveCheckResult             = (*secboot.PreinstallCheckContext).SaveCheckResult
+	secbootCheckResult                 = (*secboot.PreinstallCheckContext).CheckResult
+	secbootFDEOpteeTAPresent           = secboot.FDEOpteeTAPresent
 	preinstallCheckTimeout             = 2 * time.Minute
 
 	sysconfigConfigureTargetSystem = sysconfig.ConfigureTargetSystem
@@ -175,8 +190,15 @@ func BuildKernelBootInfo(kernInfo *snap.Info, compSeedInfos []ComponentSeedInfo,
 	}
 }
 
-// MockSecbootCheckTPMKeySealingSupported mocks secboot.CheckTPMKeySealingSupported usage by the package for testing.
+// SetAvailabilityCheckContext is a test only helper for populating EncryptionSupportInfo field availabilityCheckContext.
+func (esi *EncryptionSupportInfo) SetAvailabilityCheckContext(checkContext *secboot.PreinstallCheckContext) {
+	osutil.MustBeTestBinary("secbootPreinstallCheck can only be used in tests")
+	esi.availabilityCheckContext = checkContext
+}
+
+// MockSecbootCheckTPMKeySealingSupported mocks secbootCheckTPMKeySealingSupported usage by the package for testing.
 func MockSecbootCheckTPMKeySealingSupported(f func(tpmMode secboot.TPMProvisionMode) error) (restore func()) {
+	osutil.MustBeTestBinary("secbootCheckTPMKeySealingSupported can only be mocked in tests")
 	old := secbootCheckTPMKeySealingSupported
 	secbootCheckTPMKeySealingSupported = f
 	return func() {
@@ -184,9 +206,9 @@ func MockSecbootCheckTPMKeySealingSupported(f func(tpmMode secboot.TPMProvisionM
 	}
 }
 
-// MockSecbootPreinstallCheck mocks secboot.PreinstallCheck usage by the package for testing.
-func MockSecbootPreinstallCheck(f func(ctx context.Context, bootImagePaths []string) ([]secboot.PreinstallErrorDetails, error)) (restore func()) {
-	osutil.MustBeTestBinary("secbootPreinstallCheck only can be mocked in tests")
+// MockSecbootPreinstallCheck mocks secbootPreinstallCheck usage by the package for testing.
+func MockSecbootPreinstallCheck(f func(ctx context.Context, bootImagePaths []string) (*secboot.PreinstallCheckContext, []secboot.PreinstallErrorDetails, error)) (restore func()) {
+	osutil.MustBeTestBinary("secbootPreinstallCheck can only be mocked in tests")
 	old := secbootPreinstallCheck
 	secbootPreinstallCheck = f
 	return func() {
@@ -194,46 +216,60 @@ func MockSecbootPreinstallCheck(f func(ctx context.Context, bootImagePaths []str
 	}
 }
 
-func checkPassphraseSupportedByTargetSystem(sysVer *SystemSnapdVersions) (bool, error) {
-	const minSnapdVersion = "2.68"
-	if sysVer == nil {
-		return false, nil
-	}
-	if sysVer.SnapdVersion == "" || sysVer.SnapdInitramfsVersion == "" {
-		return false, nil
+// MockSecbootPreinstallCheckAction mocks secbootPreinstallCheckAction usage by the package for testing.
+func MockSecbootPreinstallCheckAction(f func(pcc *secboot.PreinstallCheckContext, ctx context.Context, action *secboot.PreinstallAction) ([]secboot.PreinstallErrorDetails, error)) (restore func()) {
+	osutil.MustBeTestBinary("secbootPreinstallCheckAction can only be mocked in tests")
+	old := secbootPreinstallCheckAction
+	secbootPreinstallCheckAction = f
+	return func() {
+		secbootPreinstallCheckAction = old
 	}
+}
 
-	// snapd snap must support passphrases.
-	cmp, err := strutil.VersionCompare(sysVer.SnapdVersion, minSnapdVersion)
-	if err != nil {
-		return false, fmt.Errorf("invalid snapd version in info file from snapd snap: %v", err)
-	}
-	if cmp < 0 {
-		return false, nil
-	}
-	// snap-bootstrap inside the kernel must support passphrases.
-	cmp, err = strutil.VersionCompare(sysVer.SnapdInitramfsVersion, minSnapdVersion)
-	if err != nil {
-		return false, fmt.Errorf("invalid snapd version in info file from kernel snap: %v", err)
-	}
-	if cmp < 0 {
-		return false, nil
+// MockSecbootSaveCheckResult mocks secbootSaveCheckResult usage by the package for testing.
+func MockSecbootSaveCheckResult(f func(pcc *secboot.PreinstallCheckContext, filename string) error) (restore func()) {
+	osutil.MustBeTestBinary("secbootSaveCheckResult can only be mocked in tests")
+	old := secbootSaveCheckResult
+	secbootSaveCheckResult = f
+	return func() {
+		secbootSaveCheckResult = old
 	}
+}
 
-	return true, nil
+// MockSecbootCheckResult mocks secbootCheckResult usage by the package for testing.
+func MockSecbootCheckResult(f func(pcc *secboot.PreinstallCheckContext) (*secboot.PreinstallCheckResult, error)) (restore func()) {
+	osutil.MustBeTestBinary("secbootCheckResult can only be mocked in tests")
+	old := secbootCheckResult
+	secbootCheckResult = f
+	return func() {
+		secbootCheckResult = old
+	}
+}
+
+// EncryptionConstraints is the set of constraints that
+// [GetEncryptionSupportInfo] must consider when deciding how to encrypt the
+// system.
+type EncryptionConstraints struct {
+	Model         *asserts.Model
+	Kernel        *snap.Info
+	Gadget        *gadget.Info
+	TPMMode       secboot.TPMProvisionMode
+	SnapdVersions SystemSnapdVersions
+	CheckContext  *secboot.PreinstallCheckContext
+	CheckAction   *secboot.PreinstallAction
 }
 
 // GetEncryptionSupportInfo returns the encryption support information
 // for the given model, TPM provision mode, kernel and gadget information and
 // system hardware. It uses runSetupHook to invoke the kernel fde-setup hook if
 // any is available, leaving the caller to decide how, based on the environment.
-func GetEncryptionSupportInfo(model *asserts.Model, tpmMode secboot.TPMProvisionMode, kernelInfo *snap.Info, gadgetInfo *gadget.Info, systemSnapdVersions *SystemSnapdVersions, runSetupHook fde.RunSetupHookFunc) (EncryptionSupportInfo, error) {
-	secured := model.Grade() == asserts.ModelSecured
-	dangerous := model.Grade() == asserts.ModelDangerous
-	encrypted := model.StorageSafety() == asserts.StorageSafetyEncrypted
+func GetEncryptionSupportInfo(constraints EncryptionConstraints, runSetupHook fde.RunSetupHookFunc) (EncryptionSupportInfo, error) {
+	secured := constraints.Model.Grade() == asserts.ModelSecured
+	dangerous := constraints.Model.Grade() == asserts.ModelDangerous
+	encrypted := constraints.Model.StorageSafety() == asserts.StorageSafetyEncrypted
 
 	res := EncryptionSupportInfo{
-		StorageSafety: model.StorageSafety(),
+		StorageSafety: constraints.Model.StorageSafety(),
 	}
 
 	// check if we should disable encryption non-secured devices
@@ -243,21 +279,37 @@ func GetEncryptionSupportInfo(model *asserts.Model, tpmMode secboot.TPMProvision
 		return res, nil
 	}
 
-	// check encryption: this can either be provided by the fde-setup
-	// hook mechanism or by the built-in secboot based encryption
-	checkFDESetupHookEncryption := hasFDESetupHookInKernel(kernelInfo)
-	// Note that having a fde-setup hook will disable the internal
-	// secboot based encryption
-	checkSecbootEncryption := !checkFDESetupHookEncryption
+	// check encryption: this can either be provided by one of three mechanisms,
+	// and they are checked in this order:
+	// - the fde-setup hook
+	// - the optee trusted application
+	// - the built-in secboot based encryption
+
+	checkFDESetupHookEncryption := hasFDESetupHookInKernel(constraints.Kernel)
+	// unlike the fde-setup hook, we don't have a way to statically verify that
+	// we should use optee. thus, we check for its presence and fall back to the
+	// tpm if we can't see optee. this means that this function won't return any
+	// optee-specific errors/warnings. change this?
+	checkOPTEEEncryption := !checkFDESetupHookEncryption && secbootFDEOpteeTAPresent()
+	checkSecbootEncryption := !checkOPTEEEncryption
+
 	var checkEncryptionErr error
 	switch {
 	case checkFDESetupHookEncryption:
 		res.Type, checkEncryptionErr = checkFDEFeatures(runSetupHook)
+	case checkOPTEEEncryption:
+		res.Type = device.EncryptionTypeLUKS
 	case checkSecbootEncryption:
-		unavailableReason, preinstallErrorDetails, err := encryptionAvailabilityCheck(model, tpmMode)
+		preinstallCheckContext, unavailableReason, preinstallErrorDetails, err := encryptionAvailabilityCheck(
+			constraints.CheckContext,
+			constraints.CheckAction,
+			constraints.Model,
+			constraints.TPMMode,
+		)
 		if err != nil {
 			return res, fmt.Errorf("internal error: cannot perform secboot encryption check: %v", err)
 		}
+		res.availabilityCheckContext = preinstallCheckContext
 
 		if unavailableReason == "" {
 			res.Type = device.EncryptionTypeLUKS
@@ -293,16 +345,16 @@ func GetEncryptionSupportInfo(model *asserts.Model, tpmMode secboot.TPMProvision
 		// it is usually in the context of embedded systems where passphrase
 		// authentication is not practical.
 		if checkSecbootEncryption {
-			passphraseAuthAvailable, err := checkPassphraseSupportedByTargetSystem(systemSnapdVersions)
-			if err != nil {
-				return res, fmt.Errorf("cannot check passphrase support: %v", err)
-			}
-			res.PassphraseAuthAvailable = passphraseAuthAvailable
+			// TODO:FDEM: re-enable PIN and passphrase support during install
+			// after we figure out how to properly obtain the keyboard layout
+			// during install-time for plymouth.
+			res.PassphraseAuthAvailable = false
+			res.PINAuthAvailable = false
 		}
 		opts := &gadget.ValidationConstraints{
 			EncryptedData: true,
 		}
-		if err := gadget.Validate(gadgetInfo, model, opts); err != nil {
+		if err := gadget.Validate(constraints.Gadget, constraints.Model, opts); err != nil {
 			if secured || encrypted {
 				res.UnavailableErr = fmt.Errorf("cannot use encryption with the gadget: %v", err)
 			} else {
@@ -316,44 +368,65 @@ func GetEncryptionSupportInfo(model *asserts.Model, tpmMode secboot.TPMProvision
 	return res, nil
 }
 
-func encryptionAvailabilityCheck(model *asserts.Model, tpmMode secboot.TPMProvisionMode) (string, []secboot.PreinstallErrorDetails, error) {
+func encryptionAvailabilityCheck(
+	checkContext *secboot.PreinstallCheckContext,
+	checkAction *secboot.PreinstallAction,
+	model *asserts.Model,
+	tpmMode secboot.TPMProvisionMode,
+) (*secboot.PreinstallCheckContext, string, []secboot.PreinstallErrorDetails, error) {
 	supported, err := preinstallCheckSupportedWithEnvFallback(model)
 	if err != nil {
-		return "", nil, fmt.Errorf("cannot confirm preinstall check support: %v", err)
+		return nil, "", nil, fmt.Errorf("cannot confirm preinstall check support: %v", err)
 	}
 	if supported {
 		// use comprehensive preinstall check
 		images, err := orderedCurrentBootImages(model)
 		if err != nil {
-			return "", nil, fmt.Errorf("cannot locate ordered current boot images: %v", err)
+			return nil, "", nil, fmt.Errorf("cannot locate ordered current boot images: %v", err)
+		}
+
+		if checkAction != nil && checkContext == nil {
+			return nil, "", nil, errors.New("cannot use preinstall check action without context")
 		}
 
 		ctx, cancel := context.WithTimeout(context.Background(), preinstallCheckTimeout)
 		defer cancel()
-		preinstallErrorDetails, err := secbootPreinstallCheck(ctx, images)
+
+		var (
+			preinstallErrorDetails []secboot.PreinstallErrorDetails
+			newCheckContext        *secboot.PreinstallCheckContext
+		)
+
+		if checkContext != nil {
+			preinstallErrorDetails, err = secbootPreinstallCheckAction(checkContext, ctx, checkAction)
+			newCheckContext = checkContext
+		} else {
+			newCheckContext, preinstallErrorDetails, err = secbootPreinstallCheck(ctx, images)
+		}
+
 		if err != nil {
 			if errors.Is(err, context.DeadlineExceeded) {
-				return "", nil, fmt.Errorf("preinstall check timed out: %v", err)
+				return nil, "", nil, fmt.Errorf("preinstall check timed out: %v", err)
 			}
-			return "", nil, err
+			return nil, "", nil, err
 		}
 
 		switch len(preinstallErrorDetails) {
 		case 0:
-			return "", nil, nil
+			return newCheckContext, "", nil, nil
 		case 1:
-			return preinstallErrorDetails[0].Message, preinstallErrorDetails, nil
+			return newCheckContext, preinstallErrorDetails[0].Message, preinstallErrorDetails, nil
 		default:
-			return fmt.Sprintf("preinstall check identified %d errors", len(preinstallErrorDetails)), preinstallErrorDetails, nil
+			return newCheckContext, fmt.Sprintf("preinstall check identified %d errors", len(preinstallErrorDetails)), preinstallErrorDetails, nil
 		}
 	}
 
 	// use general availability check
 	err = secbootCheckTPMKeySealingSupported(tpmMode)
 	if err != nil {
-		return err.Error(), nil, nil
+		return nil, err.Error(), nil, nil
 	}
-	return "", nil, nil
+	return nil, "", nil, nil
 }
 
 func preinstallCheckSupportedWithEnvFallback(model *asserts.Model) (bool, error) {
@@ -363,13 +436,13 @@ func preinstallCheckSupportedWithEnvFallback(model *asserts.Model) (bool, error)
 		return false, nil
 	}
 
-	return CheckHybridPluckyRelease(model)
+	return CheckHybridQuestingRelease(model)
 }
 
-// CheckHybridPluckyRelease returns true if the given model and runtime release
+// CheckHybridQuestingRelease returns true if the given model and runtime release
 // information indicates that this is a hybrid Ubuntu system with a version of
 // 25.10 or higher.
-func CheckHybridPluckyRelease(model *asserts.Model) (bool, error) {
+func CheckHybridQuestingRelease(model *asserts.Model) (bool, error) {
 	if !model.HybridClassic() {
 		return false, nil
 	}
@@ -456,13 +529,10 @@ func checkFDEFeatures(runSetupHook fde.RunSetupHookFunc) (et device.EncryptionTy
 // available if any and returns the corresponding device.EncryptionType,
 // internally it uses GetEncryptionSupportInfo with the provided parameters.
 func CheckEncryptionSupport(
-	model *asserts.Model,
-	tpmMode secboot.TPMProvisionMode,
-	kernelInfo *snap.Info,
-	gadgetInfo *gadget.Info,
+	constraints EncryptionConstraints,
 	runSetupHook fde.RunSetupHookFunc,
 ) (device.EncryptionType, error) {
-	res, err := GetEncryptionSupportInfo(model, tpmMode, kernelInfo, gadgetInfo, nil, runSetupHook)
+	res, err := GetEncryptionSupportInfo(constraints, runSetupHook)
 	if err != nil {
 		return "", err
 	}
@@ -508,8 +578,11 @@ func BuildInstallObserver(model *asserts.Model, gadgetDir string, useEncryption
 // It is the responsibility of the caller to call
 // ObserveExistingTrustedRecoveryAssets on trustedInstallObserver.
 func PrepareEncryptedSystemData(
-	model *asserts.Model, installKeyForRole map[string]secboot.BootstrappedContainer,
-	volumesAuth *device.VolumesAuthOptions, trustedInstallObserver boot.TrustedAssetsInstallObserver,
+	model *asserts.Model,
+	installKeyForRole map[string]secboot.BootstrappedContainer,
+	volumesAuth *device.VolumesAuthOptions,
+	checkContext *secboot.PreinstallCheckContext,
+	trustedInstallObserver boot.TrustedAssetsInstallObserver,
 ) error {
 	// validity check
 	if len(installKeyForRole) == 0 || installKeyForRole[gadget.SystemData] == nil || installKeyForRole[gadget.SystemSave] == nil {
@@ -567,17 +640,39 @@ func PrepareEncryptedSystemData(
 			}
 		}
 	}
+
+	// the check result contains information required for sealing and resealing
+	// using the optimum PCR configuration determined during preinstall check
+	var checkResult *secboot.PreinstallCheckResult
+	// non nil checkContext means that a preinstall check was performed
+	if checkContext != nil {
+		var err error
+		if checkResult, err = secbootCheckResult(checkContext); err != nil {
+			return err
+		}
+		// write the check result to file to make it available post-install for
+		// resealing operations
+		if err = saveCheckResult(checkContext); err != nil {
+			return err
+		}
+	}
+
 	// write markers containing a secret to pair data and save
 	if err := writeMarkers(model); err != nil {
 		return err
 	}
 
-	// make note of the encryption keys and auth options
-	trustedInstallObserver.SetEncryptionParams(dataBootstrappedContainer, saveBootstrappedContainer, primaryKey, volumesAuth)
+	// make note of the encryption keys and auth options, and the check result
+	trustedInstallObserver.SetEncryptionParams(dataBootstrappedContainer, saveBootstrappedContainer, primaryKey, volumesAuth, checkResult)
 
 	return nil
 }
 
+func saveCheckResult(checkContext *secboot.PreinstallCheckContext) error {
+	saveCheckResultPath := device.PreinstallCheckResultUnder(boot.InstallHostFDESaveDir)
+	return secbootSaveCheckResult(checkContext, saveCheckResultPath)
+}
+
 // writeMarkers writes markers containing the same secret to pair data and save.
 func writeMarkers(model *asserts.Model) error {
 	// ensure directory for markers exists
