@@ -55,6 +55,7 @@ import (
 	"github.com/snapcore/snapd/overlord/snapshotstate"
 	"github.com/snapcore/snapd/overlord/snapstate"
 	_ "github.com/snapcore/snapd/overlord/snapstate/policy"
+	"github.com/snapcore/snapd/release"
 
 	// import to register linkNotify callback
 	_ "github.com/snapcore/snapd/overlord/snapstate/agentnotify"
@@ -77,8 +78,6 @@ var (
 
 	pruneMaxChanges = 500
 
-	defaultCachedDownloads = 5
-
 	configstateInit = configstate.Init
 	systemdSdNotify = systemd.SdNotify
 )
@@ -191,6 +190,7 @@ func New(restartHandler restart.Handler) (*Overlord, error) {
 	if err != nil {
 		return nil, err
 	}
+	deviceMgr.AddOnInit(fdeMgr)
 	o.addManager(deviceMgr)
 
 	o.addManager(clusterstate.Manager(s))
@@ -361,7 +361,12 @@ func (o *Overlord) newStoreWithContext(storeCtx store.DeviceAndAuthContext) snap
 	cfg := store.DefaultConfig()
 	cfg.Proxy = o.proxyConf
 	sto := storeNew(cfg, storeCtx)
-	sto.SetCacheDownloads(defaultCachedDownloads)
+	// TODO add a way for overriding cache policy
+	if release.OnClassic {
+		sto.SetCachePolicy(store.DefaultCachePolicyClassic)
+	} else {
+		sto.SetCachePolicy(store.DefaultCachePolicyCore)
+	}
 	return sto
 }
 
