@@ -149,7 +149,7 @@ func (t *target) setups(st *state.State, opts Options) (SnapSetup, []ComponentSe
 
 	providerContentAttrs := defaultProviderContentAttrs(st, t.info, opts.PrereqTracker)
 
-	return SnapSetup{
+	snapsup := SnapSetup{
 		Channel:      t.setup.Channel,
 		CohortKey:    t.setup.CohortKey,
 		DownloadInfo: t.setup.DownloadInfo,
@@ -174,7 +174,16 @@ func (t *target) setups(st *state.State, opts Options) (SnapSetup, []ComponentSe
 			// XXX we store this for the benefit of old snapd
 			Website: t.info.Website(),
 		},
-	}, compsups, nil
+	}
+
+	// TODO until dm-verity data are used for all snaps, we will only
+	// use integrity data for specific snap types (the essential snaps).
+	typ := t.info.Type()
+	if typ == snap.TypeBase || typ == snap.TypeKernel || typ == snap.TypeGadget || typ == snap.TypeSnapd {
+		snapsup.IntegrityDataInfo = t.setup.IntegrityDataInfo
+	}
+
+	return snapsup, compsups, nil
 }
 
 // InstallGoal represents a single snap or a group of snaps to be installed.
@@ -303,9 +312,10 @@ func (s *storeInstallGoal) toInstall(ctx context.Context, st *state.State, opts
 
 		installs = append(installs, target{
 			setup: SnapSetup{
-				DownloadInfo: &r.DownloadInfo,
-				Channel:      channel,
-				CohortKey:    sn.RevOpts.CohortKey,
+				DownloadInfo:      &r.DownloadInfo,
+				Channel:           channel,
+				CohortKey:         sn.RevOpts.CohortKey,
+				IntegrityDataInfo: r.IntegrityData,
 			},
 			info:       r.Info,
 			snapst:     *snapst,
