@@ -40,20 +40,22 @@ import (
 
 type (
 	ResealKeysWithTPMParams = resealKeysWithTPMParams
-	PreinstallCheckResult   = preinstallCheckResult
 )
 
 var (
 	UnwrapPreinstallCheckError         = unwrapPreinstallCheckError
 	ConvertPreinstallCheckErrorType    = convertPreinstallCheckErrorType
 	ConvertPreinstallCheckErrorActions = convertPreinstallCheckErrorActions
-	Save                               = (*preinstallCheckResult).save
+	Save                               = (*PreinstallCheckResult).save
 
 	EFIImageFromBootFile = efiImageFromBootFile
 	LockTPMSealedKeys    = lockTPMSealedKeys
 
 	ResealKeysWithTPM          = resealKeysWithTPM
 	ResealKeysWithFDESetupHook = resealKeysWithFDESetupHook
+
+	EntropyBits = entropyBitsImpl
+	ValidatePIN = validatePINImpl
 )
 
 func ExtractSbRunChecksContext(checkContext *PreinstallCheckContext) *sb_preinstall.RunChecksContext {
@@ -72,6 +74,10 @@ func MockSbPreinstallNewRunChecksContext(f func(initialFlags sb_preinstall.Check
 	}
 }
 
+func NewPreinstallCheckResult(sbcheckResult *sb_preinstall.CheckResult, sbPCRProfileOptions sb_preinstall.PCRProfileOptionsFlags) *PreinstallCheckResult {
+	return &PreinstallCheckResult{sbcheckResult, sbPCRProfileOptions}
+}
+
 func MockSbPreinstallRun(f func(checkCtx *sb_preinstall.RunChecksContext, ctx context.Context, action sb_preinstall.Action, args map[string]json.RawMessage) (*sb_preinstall.CheckResult, error)) (restore func()) {
 	old := sbPreinstallRunChecks
 	sbPreinstallRunChecks = f
@@ -154,15 +160,6 @@ func MockSbBlockPCRProtectionPolicies(f func(tpm *sb_tpm2.Connection, pcrs []int
 	}
 }
 
-func MockSbActivateVolumeWithRecoveryKey(f func(volumeName, sourceDevicePath string,
-	authRequester sb.AuthRequestor, options *sb.ActivateVolumeOptions) error) (restore func()) {
-	old := sbActivateVolumeWithRecoveryKey
-	sbActivateVolumeWithRecoveryKey = f
-	return func() {
-		sbActivateVolumeWithRecoveryKey = old
-	}
-}
-
 func MockSbActivateVolumeWithKey(f func(volumeName, sourceDevicePath string, key []byte,
 	options *sb.ActivateVolumeOptions) error) (restore func()) {
 	old := sbActivateVolumeWithKey
@@ -172,14 +169,6 @@ func MockSbActivateVolumeWithKey(f func(volumeName, sourceDevicePath string, key
 	}
 }
 
-func MockSbActivateVolumeWithKeyData(f func(volumeName, sourceDevicePath string, authRequestor sb.AuthRequestor, options *sb.ActivateVolumeOptions, keys ...*sb.KeyData) error) (restore func()) {
-	oldSbActivateVolumeWithKeyData := sbActivateVolumeWithKeyData
-	sbActivateVolumeWithKeyData = f
-	return func() {
-		sbActivateVolumeWithKeyData = oldSbActivateVolumeWithKeyData
-	}
-}
-
 func MockSbMeasureSnapSystemEpochToTPM(f func(tpm *sb_tpm2.Connection, pcrIndex int) error) (restore func()) {
 	old := sbMeasureSnapSystemEpochToTPM
 	sbMeasureSnapSystemEpochToTPM = f
@@ -281,6 +270,14 @@ func MockSbNewTPMPassphraseProtectedKey(f func(tpm *sb_tpm2.Connection, params *
 	}
 }
 
+func MockSbNewTPMPINProtectedKey(f func(tpm *sb_tpm2.Connection, params *sb_tpm2.PINProtectKeyParams, pin sb.PIN) (protectedKey *sb.KeyData, primaryKey sb.PrimaryKey, unlockKey sb.DiskUnlockKey, err error)) (restore func()) {
+	old := sbNewTPMPINProtectedKey
+	sbNewTPMPINProtectedKey = f
+	return func() {
+		sbNewTPMPINProtectedKey = old
+	}
+}
+
 func MockSbSetModel(f func(model sb.SnapModel)) (restore func()) {
 	old := sbSetModel
 	sbSetModel = f
@@ -554,6 +551,10 @@ func MockSbKeyDataChangePassphrase(f func(d *sb.KeyData, oldPassphrase string, n
 	return testutil.Mock(&sbKeyDataChangePassphrase, f)
 }
 
+func MockSbKeyDataChangePIN(f func(d *sb.KeyData, oldPIN sb.PIN, newPIN sb.PIN) error) (restore func()) {
+	return testutil.Mock(&sbKeyDataChangePIN, f)
+}
+
 func NewKeyData(kd *sb.KeyData) KeyData {
 	return &keyData{kd: kd}
 }
@@ -573,3 +574,47 @@ func MockSbKeyDataPlatformName(f func(d *sb.KeyData) string) (restore func()) {
 func MockSbPCRPolicyCounterHandle(f func(skd *sb_tpm2.SealedKeyData) tpm2.Handle) (restore func()) {
 	return testutil.Mock(&sbPCRPolicyCounterHandle, f)
 }
+
+func MockSbFindStorageContainer(f func(ctx context.Context, path string) (sb.StorageContainer, error)) (restore func()) {
+	return testutil.Mock(&sbFindStorageContainer, f)
+}
+
+func MockSbWithVolumeName(f func(name string) sb.ActivateOption) (restore func()) {
+	return testutil.Mock(&sbWithVolumeName, f)
+}
+
+func MockSbWithExternalKeyData(f func(name string, keyData *sb.KeyData) sb.ActivateOption) (restore func()) {
+	return testutil.Mock(&sbWithExternalKeyData, f)
+}
+
+func MockSbWithLegacyKeyringKeyDescriptionPaths(f func(paths ...string) sb.ActivateOption) (restore func()) {
+	return testutil.Mock(&sbWithLegacyKeyringKeyDescriptionPaths, f)
+}
+
+func MockSbWithRecoveryKeyTries(f func(n uint) sb.ActivateContextOption) (restore func()) {
+	return testutil.Mock(&sbWithRecoveryKeyTries, f)
+}
+
+func MockSbNewActivateContext(f func(ctx context.Context, state *sb.ActivateState, opts ...sb.ActivateContextOption) (*sb.ActivateContext, error)) (restore func()) {
+	return testutil.Mock(&sbNewActivateContext, f)
+}
+
+func MockSbWithAuthRequestor(f func(req sb.AuthRequestor) sb.ActivateContextOption) (restore func()) {
+	return testutil.Mock(&sbWithAuthRequestor, f)
+}
+
+func MockSbWithPassphraseTries(f func(n uint) sb.ActivateContextOption) (restore func()) {
+	return testutil.Mock(&sbWithPassphraseTries, f)
+}
+
+func MockSbWithPINTries(f func(n uint) sb.ActivateContextOption) (restore func()) {
+	return testutil.Mock(&sbWithPINTries, f)
+}
+
+func MockSbWithExternalUnlockKey(f func(name string, key sb.DiskUnlockKey, src sb.ExternalUnlockKeySource) sb.ActivateOption) (restore func()) {
+	return testutil.Mock(&sbWithExternalUnlockKey, f)
+}
+
+func MockSbWithAuthRequestorUserVisibleName(f func(name string) sb.ActivateOption) (restore func()) {
+	return testutil.Mock(&sbWithAuthRequestorUserVisibleName, f)
+}
