@@ -25,11 +25,16 @@ import (
 )
 
 type PreinstallCheckContext struct{}
+type PreinstallCheckResult struct{}
 
 func PreinstallCheck(ctx context.Context, bootImagePaths []string) (*PreinstallCheckContext, []PreinstallErrorDetails, error) {
 	return nil, nil, errBuildWithoutSecboot
 }
 
+func LoadCheckResult(filename string) (*PreinstallCheckResult, error) {
+	return nil, errBuildWithoutSecboot
+}
+
 func (c *PreinstallCheckContext) PreinstallCheckAction(ctx context.Context, action *PreinstallAction) ([]PreinstallErrorDetails, error) {
 	return nil, errBuildWithoutSecboot
 }
@@ -37,3 +42,7 @@ func (c *PreinstallCheckContext) PreinstallCheckAction(ctx context.Context, acti
 func (c *PreinstallCheckContext) SaveCheckResult(filename string) error {
 	return errBuildWithoutSecboot
 }
+
+func (cc *PreinstallCheckContext) CheckResult() (*PreinstallCheckResult, error) {
+	return nil, errBuildWithoutSecboot
+}
