@@ -145,6 +145,8 @@ type SealKeysParams struct {
 	TPMPolicyAuthKeyFile string
 	// Optional volume authentication options
 	VolumesAuth *device.VolumesAuthOptions
+	// Optional preinstall check result for optimum PCR configuration
+	CheckResult *PreinstallCheckResult
 	// The key role (run, run+recover, recover)
 	KeyRole string
 	// Whether to allow disabled DMA protection
@@ -193,6 +195,9 @@ type KeyData interface {
 	// ChangePassphrase changes passphrase given old passphrase.
 	// AuthMode must be device.AuthModePassphrase.
 	ChangePassphrase(oldPassphrase, newPassphrase string) error
+	// ChangePIN changes pin given old pin.
+	// AuthMode must be device.AuthModePIN.
+	ChangePIN(oldPIN, newPIN string) error
 	// WriteTokenAtomic saves this key data to the specified LUKS2 token.
 	WriteTokenAtomic(devicePath, slotName string) error
 }
@@ -252,6 +257,9 @@ type UnlockResult struct {
 	// - UnlockedWithSealedKey
 	// - UnlockedWithKey
 	UnlockMethod UnlockMethod
+	// Keyslot is the name of the keyslot used or the name of the
+	// associated keyfile used.
+	Keyslot string
 }
 
 type ProtectKeyParams struct {
@@ -322,3 +330,11 @@ type ResealKeyParams struct {
 	// Whether old ambiguous key formats interpreted as FDE hook keys.
 	HintExpectFDEHook bool
 }
+
+// LegacyKeyFile represents a legacy key file
+type LegacyKeyFile struct {
+	// Name is the value that will be reported as used keyslot
+	Name string
+	// Path is the absolute path to the key file
+	Path string
+}
