@@ -37,6 +37,7 @@ import (
 	"github.com/snapcore/snapd/metautil"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/osutil/sys"
+	"github.com/snapcore/snapd/snap/integrity"
 	"github.com/snapcore/snapd/snap/naming"
 	"github.com/snapcore/snapd/snapdtool"
 	"github.com/snapcore/snapd/strutil"
@@ -64,6 +65,17 @@ type ContainerPlaceInfo interface {
 
 	// MountDescription is the value for the mount unit Description field.
 	MountDescription() string
+
+	// DmVerityFile returns the name of the dm-verity hash file computed by the container's name
+	// and the digest.
+	// If the container doesn't contain integrity data or contains integrity data but not of type
+	// "dm-verity", this will return an error.
+	DmVerityFile() (string, error)
+
+	// DmVerityDigest returns the dm-verity digest of the integrity data associated with the container.
+	// If the container doesn't contain integrity data or contains integrity data but not of type
+	// "dm-verity", this will return an error.
+	DmVerityDigest() (string, error)
 }
 
 // PlaceInfo offers all the information about where a snap and its data are
@@ -437,6 +449,9 @@ type Info struct {
 
 	// Categories this snap is in.
 	Categories []CategoryInfo
+
+	// IntegrityData available for this snap
+	IntegrityData *IntegrityDataInfo
 }
 
 // StoreAccount holds information about a store account, for example of snap
@@ -791,10 +806,12 @@ func (s *Info) ExpandSnapVariables(path string) string {
 	// always inside the mount namespace snap-confine creates and there
 	// we will always have a /snap directory available regardless if the
 	// system we're running on supports this or not.
-	return s.expandSnapVariablesSetSnapMountDir(path, dirs.CoreSnapMountDir)
+	return s.ExpandSnapVariablesSetSnapMountDir(path, dirs.CoreSnapMountDir)
 }
 
-func (s *Info) expandSnapVariablesSetSnapMountDir(path, snapMountDir string) string {
+// ExpandSnapVariablesSetSnapMountDir resolves $SNAP, $SNAP_DATA and
+// $SNAP_COMMON in path for this snap using snapMountDir as root directory.
+func (s *Info) ExpandSnapVariablesSetSnapMountDir(path, snapMountDir string) string {
 	return os.Expand(path, func(v string) string {
 		switch v {
 		case "SNAP":
@@ -808,19 +825,6 @@ func (s *Info) expandSnapVariablesSetSnapMountDir(path, snapMountDir string) str
 	})
 }
 
-// ExpandSliceSnapVariablesInRootfs resolves $SNAP, $SNAP_DATA and $SNAP_COMMON
-// for paths, using the rootfs as mount namespace.
-func (s *Info) ExpandSliceSnapVariablesInRootfs(paths []string) []string {
-	expandedDirs := make([]string, 0, len(paths))
-	for _, dir := range paths {
-		expandedDirs = append(expandedDirs, filepath.Clean(
-			s.expandSnapVariablesSetSnapMountDir(
-				filepath.Join(dirs.GlobalRootDir, dir),
-				dirs.StripRootDir(dirs.SnapMountDir))))
-	}
-	return expandedDirs
-}
-
 // InstallDate returns the "install date" of the snap.
 //
 // If the snap is not active, it'll return nil; otherwise
@@ -997,6 +1001,10 @@ func (s *Info) DesktopPlugFileIDs() ([]string, error) {
 		if !ok {
 			return nil, errors.New(`internal error: "desktop-file-ids" must be a list of strings`)
 		}
+		if !strings.HasSuffix(desktopFileID, ".desktop") {
+			logger.Noticef("adding missing .desktop suffix to desktop file ID %s (snap %s)", desktopFileID, s.InstanceName())
+			desktopFileID = desktopFileID + ".desktop"
+		}
 		desktopFileIDs = append(desktopFileIDs, desktopFileID)
 	}
 	return desktopFileIDs, nil
@@ -1038,7 +1046,7 @@ func (s *Info) MangleDesktopFileName(desktopFile string) (string, error) {
 	// XXX: Do we want to fail if a desktop-file-ids entry doesn't
 	// have a corresponding file?
 	for _, desktopFileID := range desktopFileIDs {
-		if strings.TrimSuffix(base, ".desktop") == desktopFileID {
+		if base == desktopFileID {
 			return filepath.Join(dir, base), nil
 		}
 	}
@@ -1318,20 +1326,21 @@ type AppInfo struct {
 	CommandChain  []string
 	CommonID      string
 
-	Daemon          string
-	DaemonScope     DaemonScope
-	StopTimeout     timeout.Timeout
-	StartTimeout    timeout.Timeout
-	WatchdogTimeout timeout.Timeout
-	StopCommand     string
-	ReloadCommand   string
-	PostStopCommand string
-	RestartCond     RestartCondition
-	RestartDelay    timeout.Timeout
-	Completer       string
-	RefreshMode     string
-	StopMode        StopModeType
-	InstallMode     string
+	Daemon            string
+	DaemonScope       DaemonScope
+	StopTimeout       timeout.Timeout
+	StartTimeout      timeout.Timeout
+	WatchdogTimeout   timeout.Timeout
+	StopCommand       string
+	ReloadCommand     string
+	PostStopCommand   string
+	RestartCond       RestartCondition
+	RestartDelay      timeout.Timeout
+	SuccessExitStatus []string
+	Completer         string
+	RefreshMode       string
+	StopMode          StopModeType
+	InstallMode       string
 
 	// TODO: this should go away once we have more plumbing and can change
 	// things vs refactor
@@ -1347,14 +1356,23 @@ type AppInfo struct {
 
 	// list of other service names that this service will start after or
 	// before
-	After  []string
-	Before []string
+	After     []string
+	Before    []string
+	BindsTo   []string
+	PartOf    []string
+	Requisite []string
 
 	Timer *TimerInfo
 
 	Autostart string
 }
 
+// HasPlug returns true if this AppInfo has the requested plug.
+func (app *AppInfo) HasPlug(plug string) bool {
+	_, ok := app.Plugs[plug]
+	return ok
+}
+
 // Runnable returns a Runnable for this app.
 func (app *AppInfo) Runnable() Runnable {
 	return Runnable{
@@ -1475,7 +1493,7 @@ func (app *AppInfo) DesktopFile() string {
 	// Loop through desktop-file-ids desktop interface plug attribute in order to
 	// have deterministic output
 	for _, desktopFileID := range desktopFileIDs {
-		desktopFile := filepath.Join(dirs.SnapDesktopFilesDir, desktopFileID+".desktop")
+		desktopFile := filepath.Join(dirs.SnapDesktopFilesDir, desktopFileID)
 		if !osutil.FileExists(desktopFile) {
 			continue
 		}
@@ -2135,3 +2153,32 @@ type RefreshFailuresInfo struct {
 	// This allows for more aggressive backoff delay for snaps that fail after a reboot.
 	LastFailureSeverity RefreshFailureSeverity `json:"last-failure-severity,omitempty"`
 }
+
+// IntegrityDataInfo contains all the integrity metadata associated with a snap.
+type IntegrityDataInfo struct {
+	integrity.IntegrityDataParams `json:"params"`
+
+	DownloadInfo `json:"download-info,omitempty"`
+}
+
+// DmVerityFile returns the name of the dm-verity hash file computed by the snap name and the digest.
+// If the snap doesn't contain integrity data or contains integrity data but not of type
+// "dm-verity", this will return an error.
+func (s *Info) DmVerityFile() (string, error) {
+	if s.IntegrityData == nil || s.IntegrityData.Type != "dm-verity" {
+		return "", fmt.Errorf("internal error: dm-verity data not found for file %q", s.MountFile())
+	}
+
+	return integrity.DmVerityHashFileName(s.MountFile(), s.IntegrityData.Digest), nil
+}
+
+// DmVerityDigest returns the dm-verity digest of the integrity data associated with the snap.
+// If the snap doesn't contain integrity data or contains integrity data but not of type
+// "dm-verity", this will return an error.
+func (s *Info) DmVerityDigest() (string, error) {
+	if s.IntegrityData == nil || s.IntegrityData.Type != "dm-verity" {
+		return "", fmt.Errorf("internal error: dm-verity data not found for file %q", s.MountFile())
+	}
+
+	return s.IntegrityData.Digest, nil
+}
