@@ -35,6 +35,7 @@ import (
 	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/snap/integrity"
 	"github.com/snapcore/snapd/snap/naming"
 	"github.com/snapcore/snapd/snap/snapdir"
 	"github.com/snapcore/snapd/snap/snapfile"
@@ -1788,24 +1789,6 @@ func (s *infoSuite) TestExpandSnapVariables(c *C) {
 	c.Assert(info.ExpandSnapVariables("$GARBAGE/rocks"), Equals, "/rocks")
 }
 
-func (s *infoSuite) TestExpandSliceSnapVariables(c *C) {
-	dirs.SetRootDir("")
-	info, err := snap.InfoFromSnapYaml([]byte(`name: foo`))
-	c.Assert(err, IsNil)
-	info.Revision = snap.R(42)
-	c.Assert(info.ExpandSliceSnapVariablesInRootfs(
-		[]string{
-			"$SNAP/stuff",
-			"$SNAP_DATA/stuff",
-			"$SNAP_COMMON/stuff",
-			"$GARBAGE/rocks"}),
-		DeepEquals, []string{
-			filepath.Join(dirs.SnapMountDir, "foo/42/stuff"),
-			"/var/snap/foo/42/stuff",
-			"/var/snap/foo/common/stuff",
-			"/rocks"})
-}
-
 func (s *infoSuite) TestStopModeTypeKillMode(c *C) {
 	for _, t := range []struct {
 		stopMode string
@@ -2335,6 +2318,10 @@ plugs:
 
 	unscopedApps := info.AppsForPlug(unscoped)
 	c.Assert(unscopedApps, testutil.DeepUnsortedMatches, []*snap.AppInfo{info.Apps["one"], info.Apps["two"]})
+
+	appInfo := info.Apps["one"]
+	c.Assert(appInfo.HasPlug("scoped-plug"), Equals, true)
+	c.Assert(appInfo.HasPlug("non-existing-plug"), Equals, false)
 }
 
 func (s *infoSuite) TestAppsForSlot(c *C) {
@@ -2617,7 +2604,7 @@ version: 1.0
 		desktopAppYaml += "\nplugs:\n  desktop:"
 	}
 	if hasDesktopFileIDs {
-		desktopAppYaml += "\n    desktop-file-ids: [org.example, org.example.Foo]"
+		desktopAppYaml += "\n    desktop-file-ids: [org.example.desktop, org.example.Foo.desktop, org.example.Foo.WithoutDesktopSuffix]"
 	}
 	info, err := snap.InfoFromSnapYaml([]byte(desktopAppYaml))
 	c.Assert(err, IsNil)
@@ -2629,7 +2616,7 @@ version: 1.0
 	c.Assert(err, IsNil)
 
 	if hasDesktopFileIDs {
-		c.Assert(desktopFileIDs, DeepEquals, []string{"org.example", "org.example.Foo"})
+		c.Assert(desktopFileIDs, DeepEquals, []string{"org.example.desktop", "org.example.Foo.desktop", "org.example.Foo.WithoutDesktopSuffix.desktop"})
 	} else {
 		c.Assert(desktopFileIDs, IsNil)
 	}
@@ -2799,3 +2786,63 @@ func (s *infoSuite) TestDesktopFilesFromInstalledSnapMangled(c *C) {
 	const mangle = true
 	s.testDesktopFilesFromInstalledSnap(c, mangle)
 }
+
+func (s *infoSuite) TestDmVerityParamsIfPresent(c *C) {
+	info, err := snap.InfoFromSnapYaml([]byte(`name: foo
+apps:
+   foo:
+   bar:
+`))
+	c.Assert(err, IsNil)
+
+	info.IntegrityData = &snap.IntegrityDataInfo{
+		IntegrityDataParams: integrity.IntegrityDataParams{
+			Type:   "dm-verity",
+			Digest: "aaa",
+		},
+	}
+
+	expected_hash_file := integrity.DmVerityHashFileName(info.MountFile(), "aaa")
+
+	digest, err := info.DmVerityDigest()
+	c.Check(err, IsNil)
+	c.Check(digest, Equals, "aaa")
+
+	dmverity_file, err := info.DmVerityFile()
+	c.Check(err, IsNil)
+	c.Check(dmverity_file, Equals, expected_hash_file)
+}
+
+func (s *infoSuite) TestDmVerityParamsIfPresentErrors(c *C) {
+	info, err := snap.InfoFromSnapYaml([]byte(`name: foo
+apps:
+   foo:
+   bar:
+`))
+	c.Assert(err, IsNil)
+
+	// Error if IntegrityData is nil
+	digest, err := info.DmVerityDigest()
+	c.Check(digest, Equals, "")
+	c.Check(err, ErrorMatches, fmt.Sprintf("internal error: dm-verity data not found for file %q", info.MountFile()))
+
+	dmverity_file, err := info.DmVerityFile()
+	c.Check(dmverity_file, Equals, "")
+	c.Check(err, ErrorMatches, fmt.Sprintf("internal error: dm-verity data not found for file %q", info.MountFile()))
+
+	// Error if IntegrityData is not of type "dm-verity"
+	info.IntegrityData = &snap.IntegrityDataInfo{
+		IntegrityDataParams: integrity.IntegrityDataParams{
+			Type:   "some type",
+			Digest: "aaa",
+		},
+	}
+	// Error if IntegrityData is nil
+	digest, err = info.DmVerityDigest()
+	c.Check(digest, Equals, "")
+	c.Check(err, ErrorMatches, fmt.Sprintf("internal error: dm-verity data not found for file %q", info.MountFile()))
+
+	dmverity_file, err = info.DmVerityFile()
+	c.Check(dmverity_file, Equals, "")
+	c.Check(err, ErrorMatches, fmt.Sprintf("internal error: dm-verity data not found for file %q", info.MountFile()))
+}
