@@ -19,9 +19,13 @@
 
 package snapdtool
 
+import (
+	"github.com/snapcore/snapd/testutil"
+)
+
 var (
-	DistroSupportsReExec     = distroSupportsReExec
-	SystemSnapSupportsReExec = systemSnapSupportsReExec
+	CandidateVersionNewer = candidateVersionNewer
+	ExeAndRoot            = exeAndRoot
 )
 
 func MockCoreSnapdPaths(newCoreSnap, newSnapdSnap string) func() {
@@ -50,3 +54,7 @@ func MockSyscallExec(f func(argv0 string, argv []string, envv []string) (err err
 		syscallExec = oldSyscallExec
 	}
 }
+
+func MockElfInterp(f func(string) (string, error)) (restore func()) {
+	return testutil.Mock(&elfInterp, f)
+}
