@@ -965,15 +965,26 @@ func (s *Store) downloadAndApplyDelta(name, targetPath string, downloadInfo *sna
 	return nil
 }
 
-func (s *Store) CacheDownloads() int {
-	return s.cfg.CacheDownloads
+// CacheDownloads returns the configured cache policy.
+func (s *Store) CachePolicy() CachePolicy {
+	return s.cfg.CachePolicy
 }
 
-func (s *Store) SetCacheDownloads(fileCount int) {
-	s.cfg.CacheDownloads = fileCount
-	if fileCount > 0 {
-		s.cacher = NewCacheManager(dirs.SnapDownloadCacheDir, fileCount)
+// SetCachePolicy configures the snap downloads cache mechanism. With a zero
+// value policy, no downloads are cached.
+func (s *Store) SetCachePolicy(policy CachePolicy) {
+	s.cfg.CachePolicy = policy
+	zero := CachePolicy{}
+	if policy != zero {
+		s.cacher = NewCacheManager(dirs.SnapDownloadCacheDir, policy)
 	} else {
 		s.cacher = &nullCache{}
 	}
 }
+
+// CleanDownloadsCache attempts cleanup of snap downloads cache.
+//
+// Returns ErrCleanupBusy if the cache was locked for other operations.
+func (s *Store) CleanDownloadsCache() error {
+	return s.cacher.Cleanup()
+}
