@@ -317,6 +317,63 @@ func postSystemsInstallFinish(cli *client.Client,
 	return waitChange(chgId)
 }
 
+// setupPreseedChrootMounts prepares the required chroot mountpoints for preseeding.
+func setupPreseedChrootMounts(chroot string) (func(), error) {
+	var mounts []string
+	cleanup := func() {
+		for i := len(mounts) - 1; i >= 0; i-- {
+			if output, stderr, err := osutil.RunSplitOutput("umount", mounts[i]); err != nil {
+				err = osutil.OutputErrCombine(output, stderr, err)
+				logger.Noticef("error: cannot unmount %q: %v", mounts[i], err)
+			}
+		}
+	}
+
+	targets := []string{
+		"/dev",
+		"/proc",
+		"/sys",
+		"/sys/kernel/security",
+	}
+
+	for _, t := range targets {
+		destination := filepath.Join(chroot, t)
+		if err := os.MkdirAll(destination, 0755); err != nil {
+			return nil, err
+		}
+
+		args := []string{"--bind", t, destination}
+		if output, stderr, err := osutil.RunSplitOutput("mount", args...); err != nil {
+			cleanup()
+			return nil, osutil.OutputErrCombine(output, stderr, err)
+		}
+
+		mounts = append(mounts, filepath.Join(chroot, t))
+	}
+
+	return cleanup, nil
+}
+
+func postSystemsInstallPreseed(cli *client.Client, details *client.SystemDetails, root string) error {
+	cleanup, err := setupPreseedChrootMounts(root)
+	if err != nil {
+		return err
+	}
+	defer cleanup()
+
+	opts := &client.InstallSystemOptions{
+		Step:       client.InstallStepPreseed,
+		TargetRoot: &root,
+	}
+	chgID, err := cli.InstallSystem(details.Label, opts)
+	if err != nil {
+		return err
+	}
+	fmt.Printf("Change %s created\n", chgID)
+
+	return waitChange(chgID)
+}
+
 func maybeGetOptionalInstall(path string) (*client.OptionalInstallRequest, error) {
 	if path == "" {
 		return nil, nil
@@ -411,8 +468,11 @@ func unmountFilesystems(mntPts []string) (err error) {
 func createClassicRootfsIfNeeded(rootfsCreator string) error {
 	dst := runMntFor("ubuntu-data")
 
-	if output, stderr, err := osutil.RunSplitOutput(rootfsCreator, dst); err != nil {
-		return osutil.OutputErrCombine(output, stderr, err)
+	cmd := exec.Command(rootfsCreator, dst)
+	cmd.Stdout = os.Stdout
+	cmd.Stderr = os.Stderr
+	if err := cmd.Run(); err != nil {
+		return err
 	}
 
 	return nil
@@ -552,7 +612,7 @@ func fillPartiallyDefinedVolume(vol *gadget.Volume, bootDevice string) error {
 	return nil
 }
 
-func run(seedLabel, bootDevice, rootfsCreator, optionalInstallPath, recoveryKeyOut string, volumesAuth volumeAuthOptions) error {
+func run(seedLabel, bootDevice, rootfsCreator, optionalInstallPath, recoveryKeyOut string, preseedRootfs bool, volumesAuth volumeAuthOptions) error {
 	logger.Noticef("installing on %q", bootDevice)
 
 	cli := client.New(nil)
@@ -597,31 +657,45 @@ func run(seedLabel, bootDevice, rootfsCreator, optionalInstallPath, recoveryKeyO
 			}
 		}
 	}
+	logger.Noticef("creating and mounting filesystems")
+
 	mntPts, err := createAndMountFilesystems(bootDevice, details.Volumes, encryptedDevices)
 	if err != nil {
 		return fmt.Errorf("cannot create filesystems: %v", err)
 	}
 
 	hasSystemSeed := checkForRole(details, gadget.SystemSeed)
+	logger.Noticef("has system seed %v", hasSystemSeed)
 	isCore := rootfsCreator == ""
 	if isCore || !hasSystemSeed {
+		logger.Noticef("copying seed to partition")
 		if err := copySeedToDataPartition(); err != nil {
 			return fmt.Errorf("cannot create seed on data partition: %v", err)
 		}
 	}
 
 	if !isCore {
+		logger.Noticef("creating classic rootfs")
 		if err := createClassicRootfsIfNeeded(rootfsCreator); err != nil {
 			return fmt.Errorf("cannot create classic rootfs: %v", err)
 		}
 	}
 
+	if preseedRootfs {
+		logger.Noticef("preseeding classic rootfs")
+		if err := postSystemsInstallPreseed(cli, details, runMntFor("ubuntu-data")); err != nil {
+			return fmt.Errorf("cannot preseed installed system: %w", err)
+		}
+	}
+
 	if err := unmountFilesystems(mntPts); err != nil {
 		return fmt.Errorf("cannot unmount filesystems: %v", err)
 	}
+
 	if err := postSystemsInstallFinish(cli, details, bootDevice, optionalInstallPath, dgpairs); err != nil {
 		return fmt.Errorf("cannot finalize install: %v", err)
 	}
+
 	// TODO: reboot here automatically (optional)
 
 	return nil
@@ -647,6 +721,7 @@ func main() {
 	kdfType := flag.String("kdf-type", "", "KDF type for passphrase [\"argon2id\", \"argon2i\" or \"pbkdf2\"] (optional)")
 	kdfTime := flag.Duration("kdf-time", 0, "length of time to run the KDF (optional)")
 	recoveryKeyOut := flag.String("recovery-key-out", "", "indicate that a recovery key should be created and stored at given path (optional)")
+	preseedRootfs := flag.Bool("preseed-rootfs", false, "Preseed rootfs")
 
 	flag.Parse()
 
@@ -655,6 +730,11 @@ func main() {
 		os.Exit(1)
 	}
 
+	if *preseedRootfs && *rootfsCreator == "" {
+		fmt.Fprintf(os.Stderr, "Cannot preseed rootfs for Ubuntu Core\n")
+		os.Exit(1)
+	}
+
 	logger.SimpleSetup(nil)
 
 	if *bootDevice == "auto" {
@@ -667,7 +747,7 @@ func main() {
 		kdfTime:    *kdfTime,
 	}
 
-	if err := run(*seedLabel, *bootDevice, *rootfsCreator, *optionalInstallPath, *recoveryKeyOut, volumesAuth); err != nil {
+	if err := run(*seedLabel, *bootDevice, *rootfsCreator, *optionalInstallPath, *recoveryKeyOut, *preseedRootfs, volumesAuth); err != nil {
 		fmt.Fprintf(os.Stderr, "%s\n", err)
 		os.Exit(1)
 	}
