@@ -366,6 +366,10 @@ nested_is_core_lt() {
     os.query is-ubuntu-lt "${VERSION}.04"
 }
 
+nested_is_core_26_system() {
+    os.query is-resolute
+}
+
 nested_is_core_24_system() {
     os.query is-noble
 }
@@ -485,9 +489,37 @@ nested_cleanup_env() {
     rm -rf "$(nested_get_extra_snaps_path)"
 }
 
+nested_get_core_channel() {
+    if nested_is_core_26_system; then
+        # TODO: Remove when it becomes available in the other channels
+        echo "edge"
+    else
+        echo "${NESTED_CORE_CHANNEL}"
+    fi
+}
+
+nested_get_kernel_channel() {
+    if nested_is_core_26_system; then
+        # TODO: Remove when it becomes available in the other channels
+        echo "edge"
+    else
+        echo "${NESTED_KERNEL_CHANNEL}"
+    fi
+}
+
+nested_get_gadget_channel() {
+    if nested_is_core_26_system; then
+        # TODO: Remove when it becomes available in the other channels
+        echo "edge"
+    else
+        echo "${NESTED_GADGET_CHANNEL}"
+    fi
+}
+
 nested_get_image_name_base() {
     local TYPE="$1"
-    local SOURCE="${NESTED_CORE_CHANNEL}"
+    local SOURCE
+    SOURCE="$(nested_get_core_channel)"
     local NAME="${NESTED_IMAGE_ID:-generic}"
     local VERSION
 
@@ -580,6 +612,8 @@ nested_get_version() {
         echo "22"
     elif nested_is_core_24_system; then
         echo "24"
+    elif nested_is_core_26_system; then
+        echo "26"
     fi
 }
 
@@ -607,12 +641,15 @@ nested_get_model() {
         ubuntu-22.04-arm-64)
             echo "$TESTSLIB/assertions/nested-22-arm64.model"
             ;;
-         ubuntu-24.04-64)
+        ubuntu-24.04-64)
             echo "$TESTSLIB/assertions/nested-24-amd64.model"
             ;;
         ubuntu-24.04-arm-64)
             echo "$TESTSLIB/assertions/nested-24-arm64.model"
             ;;
+        ubuntu-26.04-64)
+            echo "$TESTSLIB/assertions/nested-26-amd64.model"
+            ;;
         *)
             echo "unsupported system"
             exit 1
@@ -688,7 +725,7 @@ nested_prepare_kernel() {
                 repack_kernel_snap "$kernel_snap"
 
             elif nested_is_core_ge 20; then
-                snap download --basename=pc-kernel --channel="$version/${NESTED_KERNEL_CHANNEL}" pc-kernel
+                snap download --basename=pc-kernel --channel="$version/$(nested_get_kernel_channel)" pc-kernel
 
                 # set the unix bump time if the NESTED_* var is set,
                 # otherwise leave it empty
@@ -698,7 +735,7 @@ nested_prepare_kernel() {
                     epochBumpTime="--epoch-bump-time=$epochBumpTime"
                 fi
 
-                if nested_is_core_24_system; then
+                if nested_is_core_ge 24; then
                     # shellcheck source=tests/lib/prepare.sh
                     . "$TESTSLIB"/prepare.sh
                     uc24_build_initramfs_kernel_snap "pc-kernel.snap" "$NESTED_ASSETS_DIR" "$epochBumpTime"
@@ -747,7 +784,7 @@ nested_prepare_gadget() {
             snakeoil_key="$PWD/$key_name.key"
             snakeoil_cert="$PWD/$key_name.pem"
 
-            snap download --basename=pc --channel="$version/${NESTED_GADGET_CHANNEL}" pc
+            snap download --basename=pc --channel="$version/$(nested_get_gadget_channel)" pc
             unsquashfs -d pc-gadget pc.snap
             nested_secboot_sign_gadget pc-gadget "$snakeoil_key" "$snakeoil_cert"
             case "${NESTED_UBUNTU_SAVE:-}" in
@@ -783,7 +820,7 @@ EOF
             fi
 
             if [ -n "$NESTED_EXTRA_CMDLINE" ]; then
-                GADGET_EXTRA_CMDLINE="$GADGET_EXTRA_CMDLINE $NESTED_EXTRA_CMDLINE"
+                GADGET_EXTRA_CMDLINE="ds=nocloud $GADGET_EXTRA_CMDLINE $NESTED_EXTRA_CMDLINE"
             fi
 
             if [ -n "$GADGET_EXTRA_CMDLINE" ]; then
@@ -838,6 +875,12 @@ nested_prepare_base() {
         elif nested_is_core_24_system; then
             snap_name="core24"
             snap_id="dwTAh7MZZ01zyriOZErqd1JynQLiOGvM"
+        elif nested_is_core_26_system; then
+            snap_name="core26"
+            snap_id="cUqM61hRuZAJYmIS898Ux66VY61gBbZf"
+        else
+            echo "Unknown nested core version" >&2
+            exit 1
         fi
         output_name="${snap_name}.snap"
 
@@ -965,8 +1008,10 @@ nested_create_core_vm() {
             export SNAPPY_FORCE_SAS_URL
             UBUNTU_IMAGE_SNAP_CMD=/usr/bin/snap
             export UBUNTU_IMAGE_SNAP_CMD
-            if [ -n "$NESTED_CORE_CHANNEL" ]; then
-                UBUNTU_IMAGE_CHANNEL_ARG="--channel $NESTED_CORE_CHANNEL"
+            local core_channel
+            core_channel="$(nested_get_core_channel)"
+            if [ -n "${core_channel}" ]; then
+                UBUNTU_IMAGE_CHANNEL_ARG="--channel ${core_channel}"
             else 
                 UBUNTU_IMAGE_CHANNEL_ARG=""
             fi
@@ -978,7 +1023,7 @@ nested_create_core_vm() {
             fi
             # ubuntu-image creates sparse image files
             # shellcheck disable=SC2086
-            SNAPD_DEBUG=1 "$UBUNTU_IMAGE" snap --image-size 10G --validation=enforce \
+            SNAPD_DEBUG=1 "$UBUNTU_IMAGE" snap --image-size 10G \
                "$NESTED_MODEL" \
                 $UBUNTU_IMAGE_CHANNEL_ARG \
                 "${UBUNTU_IMAGE_PRESEED_ARGS[@]:-}" \
@@ -1204,9 +1249,10 @@ nested_force_start_vm() {
     systemctl start "$NESTED_VM"
 }
 
-nested_start_core_vm_unit() {
-    local QEMU CURRENT_IMAGE
+nested_create_vm_service() {
+    local QEMU CURRENT_IMAGE PARAM_OPT
     CURRENT_IMAGE=$1
+    PARAM_OPT="${2:-}"
     QEMU=$(nested_qemu_name)
 
     # Now qemu parameters are defined
@@ -1223,7 +1269,7 @@ nested_start_core_vm_unit() {
     elif [ "$SPREAD_BACKEND" = "qemu-nested" ] || [ "$SPREAD_BACKEND" = "garden" ]; then
         PARAM_MEM="-m ${NESTED_MEM:-2048}"
         PARAM_SMP="-smp ${NESTED_CPUS:-1}"
-    elif [ "$SPREAD_BACKEND" = "openstack-ext-ps7" ] || [ "$SPREAD_BACKEND" = "openstack-validation-ps7" ]; then
+    elif [[ "$SPREAD_BACKEND" = openstack* ]]; then
         PARAM_MEM="-m ${NESTED_MEM:-4096}"
         PARAM_SMP="-smp ${NESTED_CPUS:-2}"
     else
@@ -1414,7 +1460,16 @@ nested_start_core_vm_unit() {
         ${PARAM_MONITOR} \
         ${PARAM_USB} \
         ${PARAM_CD}  \
+        ${PARAM_OPT} \
         ${PARAM_EXTRA} " "${PARAM_REEXEC_ON_FAILURE}"
+}
+
+nested_start_core_vm_unit() {
+    local CURRENT_IMAGE
+    CURRENT_IMAGE=$1
+
+    # Now qemu parameters are defined
+    nested_create_vm_service "$CURRENT_IMAGE"
 
     local EXPECT_SHUTDOWN
     EXPECT_SHUTDOWN=${NESTED_EXPECT_SHUTDOWN:-}
@@ -1459,6 +1514,9 @@ nested_start_core_vm_unit() {
 }
 
 nested_setup_vm(){
+    local modified
+    modified=0
+
     if [ "${SNAPD_USE_PROXY:-}" = true ]; then
         nested_no_proxy="${NO_PROXY},10.0.2.2"
 
@@ -1496,18 +1554,41 @@ nested_setup_vm(){
         remote.exec "echo Environment=HTTPS_PROXY=$HTTPS_PROXY HTTP_PROXY=$HTTP_PROXY https_proxy=$HTTPS_PROXY http_proxy=$HTTP_PROXY NO_PROXY=$nested_no_proxy no_proxy=$nested_no_proxy SNAPD_USE_PROXY=$SNAPD_USE_PROXY | sudo tee -a /etc/systemd/system/snapd.service.d/proxy.conf"
         remote.exec "sudo systemctl daemon-reload"
         remote.exec "sudo systemctl start snapd.service snapd.socket"
-        remote.exec "sudo sync"
+        modified=1
     fi
     if [ -n "${NTP_SERVER:-}" ]; then
-        # Configure systemd-timesyncd to use the predefined ntp server
-        CONF_FILE="/etc/systemd/timesyncd.conf"
-        remote.exec "cp \"$CONF_FILE\" /tmp/timesyncd.conf"
-        remote.exec "sed -i -e '/^NTP=/d' -e '/^FallbackNTP=/d' /tmp/timesyncd.conf"
-        remote.exec "sed -i '/^\[Time\]/a NTP='\"$NTP_SERVER\" /tmp/timesyncd.conf"
-        remote.exec "sed -i '/^\[Time\]/a FallbackNTP=' /tmp/timesyncd.conf"
-        remote.exec "sudo cp /tmp/timesyncd.conf \"$CONF_FILE\""
-        remote.exec "sudo systemctl restart systemd-timesyncd"
-        remote.exec "sudo sync"
+        # We reconfigure both chrony and timesyncd if installed. But
+        # we only restart the one started.
+        if remote.exec "[ -d /etc/chrony/sources.d ]"; then
+            remote.exec "sudo rm /etc/chrony/sources.d/*.sources"
+            echo "pool ${NTP_SERVER} iburst maxsources 1 nts prefer" | remote.exec "sudo tee /etc/chrony/sources.d/proxy.sources"
+            # try-restart will not restart if not started. Important
+            # if both timesyncd and chrony are installed but only one is
+            # running
+            remote.exec "sudo systemctl try-restart chrony.service"
+            modified=1
+        fi
+        if remote.exec "[ -f /etc/systemd/timesyncd.conf ]"; then
+            # Configure systemd-timesyncd to use the predefined ntp server
+            CONF_FILE="/etc/systemd/timesyncd.conf"
+            remote.exec "cp \"$CONF_FILE\" /tmp/timesyncd.conf"
+            remote.exec "sed -i -e '/^NTP=/d' -e '/^FallbackNTP=/d' /tmp/timesyncd.conf"
+            remote.exec "sed -i '/^\[Time\]/a NTP='\"$NTP_SERVER\" /tmp/timesyncd.conf"
+            remote.exec "sed -i '/^\[Time\]/a FallbackNTP=' /tmp/timesyncd.conf"
+            remote.exec "sudo cp /tmp/timesyncd.conf \"$CONF_FILE\""
+            # try-restart will not restart if not started. Important
+            # if both timesyncd and chrony are installed but only one is
+            # running
+            remote.exec "sudo systemctl try-restart systemd-timesyncd"
+            modified=1
+        fi
+    fi
+
+    if [ "${modified}" != 0 ]; then
+      # Some modification have happened, before return back to a test
+      # that might do a hard reset, we need to make sure the
+      # modification are saved to disk.
+      remote.exec "sudo sync"
     fi
 }
 
@@ -1649,121 +1730,20 @@ nested_start_classic_vm() {
     IMAGE_NAME="$(nested_get_image_name classic)"
 
     if [ ! -f "$NESTED_IMAGES_DIR/$IMAGE_NAME" ] ; then
-        cp -v "$NESTED_IMAGES_DIR/$IMAGE_NAME.pristine" "$IMAGE_NAME"
+        cp -v "$NESTED_IMAGES_DIR/$IMAGE_NAME.pristine" "$NESTED_IMAGES_DIR/$IMAGE_NAME"
     fi
+
     # Give extra disk space for the image
-    qemu-img resize "$NESTED_IMAGES_DIR/$IMAGE_NAME" +2G
+    qemu-img resize "$NESTED_IMAGES_DIR/$IMAGE_NAME" +4G
 
-    # Now qemu parameters are defined
-    local PARAM_SMP PARAM_MEM
-    PARAM_SMP="-smp 1"
-    # use only 2G of RAM for qemu-nested
-    if [ "$SPREAD_BACKEND" = "google-nested" ]; then
-        PARAM_MEM="-m ${NESTED_MEM:-4096}"
-        PARAM_SMP="-smp ${NESTED_CPUS:-2}"
-    elif [ "$SPREAD_BACKEND" = "google-nested-arm" ]; then
-        PARAM_MEM="-m ${NESTED_MEM:-8192}"
-        PARAM_SMP="-smp ${NESTED_CPUS:-2}"
-    elif [ "$SPREAD_BACKEND" = "qemu-nested" ] || [ "$SPREAD_BACKEND" = "garden" ]; then
-        PARAM_MEM="-m ${NESTED_MEM:-2048}"
-        PARAM_SMP="-smp ${NESTED_CPUS:-1}"
-    elif [ "$SPREAD_BACKEND" = "openstack-ext-ps7" ] || [ "$SPREAD_BACKEND" = "openstack-validation-ps7" ]; then
-        PARAM_MEM="-m ${NESTED_MEM:-4096}"
-        PARAM_SMP="-smp ${NESTED_CPUS:-2}"        
-    else
-        echo "unknown spread backend $SPREAD_BACKEND"
-        exit 1
-    fi
-    local PARAM_DISPLAY PARAM_NETWORK PARAM_MONITOR PARAM_USB PARAM_CPU PARAM_CD PARAM_RANDOM PARAM_SNAPSHOT
-    PARAM_DISPLAY="-nographic"
-    PARAM_NETWORK="-net nic,model=virtio -net user,hostfwd=tcp::$NESTED_SSH_PORT-:22"
-    PARAM_MONITOR="-monitor tcp:127.0.0.1:$NESTED_MON_PORT,server=on,wait=off"
-    PARAM_USB="-usb"
-    PARAM_CPU=""
-    PARAM_CD="${NESTED_PARAM_CD:-}"
-    PARAM_RANDOM="-object rng-random,id=rng0,filename=/dev/urandom -device virtio-rng-pci,rng=rng0"
-    # TODO: can this be removed? we create a "pristine" copy above?
-    #PARAM_SNAPSHOT="-snapshot"
-    PARAM_SNAPSHOT=""
-    PARAM_EXTRA="${NESTED_PARAM_EXTRA:-}"
+    # HACK: convert "classic" qcow2 to raw "core" image because we need
+    # to boot with OVMF, but we do this to use shared vm code
+    qemu-img convert -f qcow2 -O raw \
+        "$NESTED_IMAGES_DIR/$IMAGE_NAME" \
+        "$NESTED_IMAGES_DIR/$IMAGE_NAME.raw"
+    mv -f "$NESTED_IMAGES_DIR/$IMAGE_NAME.raw" "$NESTED_IMAGES_DIR/$IMAGE_NAME"
 
-    # XXX: duplicated from nested core vm
-    # Set kvm attribute
-    local ATTR_KVM
-    ATTR_KVM=""
-    if nested_is_kvm_enabled; then
-        ATTR_KVM=",accel=kvm"
-        # CPU can be defined just when kvm is enabled
-        PARAM_CPU="-cpu host"
-    fi
-
-    local PARAM_MACHINE PARAM_IMAGE PARAM_SEED PARAM_SERIAL PARAM_BIOS PARAM_TPM
-    if [[ "$SPREAD_BACKEND" = google-nested* ]] || [[ "$SPREAD_BACKEND" = openstack* ]]; then
-        PARAM_MACHINE="-machine ubuntu,accel=kvm"
-        PARAM_CPU="-cpu host"
-    elif [ "$SPREAD_BACKEND" = "qemu-nested" ] || [ "$SPREAD_BACKEND" = "garden" ]; then
-        # check if we have nested kvm
-        if [ "$(cat /sys/module/kvm_*/parameters/nested)" = "1" ]; then
-            PARAM_MACHINE="-machine ubuntu${ATTR_KVM}"
-        else
-            # and if not reset kvm related parameters
-            PARAM_MACHINE=""
-            PARAM_CPU=""
-            ATTR_KVM=""
-        fi
-    else
-        echo "unknown spread backend $SPREAD_BACKEND"
-        exit 1
-    fi
-
-    PARAM_IMAGE="-drive file=$NESTED_IMAGES_DIR/$IMAGE_NAME,if=none,id=disk1 -device virtio-blk-pci,drive=disk1,bootindex=1"
-    PARAM_SEED="-drive file=$NESTED_ASSETS_DIR/seed.img,if=virtio"
-    # Open port 7777 on the host so that failures in the nested VM (e.g. to
-    # create users) can be debugged interactively via
-    # "telnet localhost 7777". Also keeps the logs
-    #
-    # XXX: should serial just be logged to stdout so that we just need
-    #      to "journalctl -u $NESTED_VM" to see what is going on ?
-    if "$QEMU" -version | grep '2\.5'; then
-        # XXX: remove once we no longer support xenial hosts
-        PARAM_SERIAL="-serial file:${NESTED_LOGS_DIR}/serial.log"
-    else
-        PARAM_SERIAL="-chardev socket,telnet=on,host=localhost,server=on,port=7777,wait=off,id=char0,logfile=${NESTED_LOGS_DIR}/serial.log,logappend=on -serial chardev:char0"
-    fi
-    PARAM_BIOS=""
-    PARAM_TPM=""
-
-    # ensure we have a log dir
-    mkdir -p "$NESTED_LOGS_DIR"
-    # save logs from previous runs
-    nested_save_serial_log
-
-    rm -rf "${NESTED_ASSETS_DIR}"/qemu-creds
-    mkdir -p "${NESTED_ASSETS_DIR}"/qemu-creds
-    if [ "${NESTED_FDE_PASSWORD+set}" = set ]; then
-        echo -n "${NESTED_FDE_PASSWORD}" >"${NESTED_ASSETS_DIR}/qemu-creds/snapd.fde.password"
-    fi
-
-    # Systemd unit is created, it is important to respect the qemu parameters 
-    # order
-    tests.systemd create-and-start-unit "$NESTED_VM" "${TESTSLIB}/qemu-runner.sh ${NESTED_ASSETS_DIR}/qemu-creds ${QEMU}  \
-        ${PARAM_SMP} \
-        ${PARAM_CPU} \
-        ${PARAM_MEM} \
-        ${PARAM_SNAPSHOT} \
-        ${PARAM_MACHINE} \
-        ${PARAM_DISPLAY} \
-        ${PARAM_NETWORK} \
-        ${PARAM_BIOS} \
-        ${PARAM_TPM} \
-        ${PARAM_RANDOM} \
-        ${PARAM_IMAGE} \
-        ${PARAM_SEED} \
-        ${PARAM_SERIAL} \
-        ${PARAM_MONITOR} \
-        ${PARAM_USB} \
-        ${PARAM_EXTRA} \
-        ${PARAM_CD} "
+    nested_create_vm_service "$NESTED_IMAGES_DIR/$IMAGE_NAME" "-drive file=$NESTED_ASSETS_DIR/seed.img,if=virtio"
 
     if ! nested_wait_vm_ready 60; then
         echo "failed to wait for the vm becomes ready to receive connections"
