@@ -70,23 +70,15 @@ create_test_user(){
 }
 
 build_deb(){
-    # debian-sid packaging is special
-    if os.query is-debian sid; then
-        if [ ! -d packaging/debian-sid ]; then
-            echo "no packaging/debian-sid/ directory "
-            echo "broken test setup"
-            exit 1
-        fi
+    if [ ! -L debian ] ; then
+        echo "no debian directory or symlink, broken test setup"
+        exit 1
+    fi
 
+    if os.query is-debian sid; then
         # remove etckeeper
         apt purge -y etckeeper
 
-        # debian has its own packaging
-        rm -f debian
-        # the debian dir must be a real dir, a symlink will make
-        # dpkg-buildpackage choke later.
-        mv packaging/debian-sid debian
-
         apt build-dep -y ./
 
         # ensure we really build without vendored packages
@@ -125,7 +117,7 @@ build_rpm() {
         distro=amzn
         release=2023
     fi
-    base_version="$(head -1 debian/changelog | awk -F '[()]' '{print $2}')"
+    base_version="$(head -1 packaging/ubuntu-16.04/changelog | awk -F '[()]' '{print $2}')"
     version="1337.$base_version"
     packaging_path=packaging/$distro-$release
     rpm_dir=$(rpm --eval "%_topdir")
@@ -176,7 +168,7 @@ build_rpm() {
 }
 
 build_arch_pkg() {
-    base_version="$(head -1 debian/changelog | awk -F '[()]' '{print $2}')"
+    base_version="$(head -1 packaging/ubuntu-16.04/changelog | awk -F '[()]' '{print $2}')"
     version="1337.$base_version"
     packaging_path=packaging/arch
 
@@ -372,22 +364,28 @@ prepare_project() {
         fi
     fi
 
-    # so is ubuntu-14.04
+    # set up debian symlink as needed
     if os.query is-trusty; then
-        if [ ! -d packaging/ubuntu-14.04 ]; then
-            echo "no packaging/ubuntu-14.04/ directory "
-            echo "broken test setup"
-            exit 1
-        fi
-
-        # 14.04 has its own packaging
-        ./generate-packaging-dir
+        # no packaging setup for 14.04, we're no longer building the packages in
+        # CI
+        :
+    elif os.query is-ubuntu; then
+        # TODO generate packaging appropriate for a given ubuntu release
+        ln -sf packaging/ubuntu-16.04 debian
+    elif os.query is-debian sid ; then
+        # debian sid has special packaging
+        ln -sf packaging/debian-sid debian
+    elif os.query is-debian; then
+        # TODO debian reuses ubuntu 16.04 packaging
+        ln -sf packaging/ubuntu-16.04 debian
+    fi
 
+    if os.query is-trusty; then
         quiet eatmydata apt-get install -y software-properties-common
 
-	# FIXME: trusty-proposed disabled because there is an inconsistency
-	#        in the trusty-proposed archive:
-	# linux-generic-lts-xenial : Depends: linux-image-generic-lts-xenial (= 4.4.0.143.124) but 4.4.0.141.121 is to be installed
+        # FIXME: trusty-proposed disabled because there is an inconsistency
+        #        in the trusty-proposed archive:
+        # linux-generic-lts-xenial : Depends: linux-image-generic-lts-xenial (= 4.4.0.143.124) but 4.4.0.141.121 is to be installed
         #echo 'deb http://archive.ubuntu.com/ubuntu/ trusty-proposed main universe' >> /etc/apt/sources.list
         quiet add-apt-repository ppa:snappy-dev/image
         quiet eatmydata apt-get update
@@ -528,26 +526,38 @@ prepare_project() {
     # base on the packaging. In Fedora/Suse this is handled via mock/osc
     case "$SPREAD_SYSTEM" in
         debian-*|ubuntu-*)
-            best_golang=golang-1.18
-            case "$SPREAD_SYSTEM" in
-                ubuntu-fips-*)
-                    # we are limited by the FIPS variants of go toolchain
-                    # available from the PPA, and we need to match the Go
-                    # version expected by during FIPS build of the deb, which
-                    # currently expects 1.23, see:
-                    # https://launchpad.net/~ubuntu-toolchain-r/+archive/ubuntu/golang-fips
-                    best_golang=golang-1.23
-                    quiet apt install -y golang-1.23
-                    ;;
-            esac
-            # in 16.04: "apt build-dep -y ./" would also work but not on 14.04
-            gdebi --quiet --apt-line ./debian/control >deps.txt
-            quiet xargs -r eatmydata apt-get install -y < deps.txt
-            # The go 1.18 backport is not using alternatives or anything else so
-            # we need to get it on path somehow. This is not perfect but simple.
-            if [ -z "$(command -v go)" ]; then
-                # the path filesystem path is: /usr/lib/go-1.18/bin
-                ln -s "/usr/lib/${best_golang/lang/}/bin/go" /usr/bin/go
+            do_depinstall() {
+                best_golang=golang-1.18
+                case "$SPREAD_SYSTEM" in
+                    ubuntu-fips-*)
+                        # we are limited by the FIPS variants of go toolchain
+                        # available from the PPA, and we need to match the Go
+                        # version expected by during FIPS build of the deb, which
+                        # currently expects 1.23, see:
+                        # https://launchpad.net/~ubuntu-toolchain-r/+archive/ubuntu/golang-fips
+                        best_golang=golang-1.23
+                        quiet apt install -y golang-1.23
+                        ;;
+                esac
+                # in 16.04: "apt build-dep -y ./" would also work but not on 14.04
+                gdebi --quiet --apt-line ./debian/control >deps.txt
+                quiet xargs -r eatmydata apt-get install -y < deps.txt
+                # The go 1.18 backport is not using alternatives or anything else so
+                # we need to get it on path somehow. This is not perfect but simple.
+                if [ -z "$(command -v go)" ]; then
+                    # the path filesystem path is: /usr/lib/go-1.18/bin
+                    ln -s "/usr/lib/${best_golang/lang/}/bin/go" /usr/bin/go
+                fi
+            }
+
+            if ! os.query is-trusty; then
+                # Debian and Ubuntu non-14.04
+                do_depinstall
+            else
+                # we only run a limited set of tests on 14.04, hence only Go is
+                # needed to build the dependencies, but that's all
+                eatmydata apt-get install -y golang-1.18
+                ln -s "/usr/lib/go-1.18/bin/go" /usr/bin/go
             fi
             ;;
     esac
@@ -559,6 +569,7 @@ prepare_project() {
     # We are testing snapd snap on top of snapd from the archive
     # of the tested distribution. Download snapd and snap-confine
     # as they exist in the archive for further use.
+    # On 14.04 we only ever use snapd from the archive
     if tests.info is-snapd-from-archive; then
         case "$SPREAD_SYSTEM" in
             debian-*)
@@ -566,6 +577,10 @@ prepare_project() {
                 # In earlier versions it was just an empty package so it's not worth pulling.
                 ( cd "${GOHOME}" && tests.pkgs download snapd )
                 ;;
+            ubuntu-14.04-*)
+                # directly call apt-get, tests.pkgs only knows about 'apt'
+                ( cd "${GOHOME}" && apt-get download snapd snap-confine )
+                ;;
             *)
                 ( cd "${GOHOME}" && tests.pkgs download snapd snap-confine)
                 ;;
@@ -586,6 +601,10 @@ prepare_project() {
         # go mod runs as root and will leave strange permissions
         chown test:test -R "$SPREAD_PATH"
         case "$SPREAD_SYSTEM" in
+            ubuntu-14.04-*)
+                echo "building native packages on 14.04 is no longer supported"
+                exit 1
+                ;;
             ubuntu-*|debian-*)
                 # in 16.04: "apt build-dep -y ./" would also work but not on 14.04
                 gdebi --quiet --apt-line ./debian/control >deps.txt
